import os,sys,numpy,re,math

def slopelist(slope,distmin,distmax,distsep):
    """
    Create array of slope values.
    
    slope : float
      Middle or single slope value
    distmin : float
      Minimum slope value
    distmax : float
      Maximum slope value
    distsep : float
      Step size value
    """
    distlist = distplot = [slope]
    if distmin < distmax <= slope:
        distlist = numpy.arange(distmax,distmin-0.001,-distsep)
        distplot = sorted(distlist)
    elif distmax > distmin >= slope:
        distlist = numpy.arange(distmin,distmax+0.001,distsep)
        distplot = sorted(distlist)
    elif distmin < slope < distmax:
        distlist = numpy.hstack((numpy.arange(slope,distmax+0.001,+distsep),
                                 numpy.arange(slope,distmin-0.001,-distsep)))
        distplot = sorted(numpy.delete(distlist,0))
    return distlist,distplot

def getshift(left,right,slope,qso=None,sample=None,explist=None,settings=None,shape=False,whitmore=False,stdev=0):
    """
    Calculate velocity shift do to distortion depending on model used.

    Parameters
    ----------
    qso : str
      Quasar name.
    sample : str
      Data sample.
    left : float
      Minimum wavelength of the fitting region
    right : float
      Maximum wavelength of the fitting region
    slope : float
      Slope of the long-range distortion model
    whitmore : bool
      Flag to do simplistic Whitmore distortion model
    stdev : float
      Standard deviation if random slope used

    Returns
    -------
    shift : float
      Velocity shift due to long-range distortion effect.
    """
    # Load files
    pathdata = os.path.dirname(os.path.realpath(__file__)) + '/data/'
    explist  = explist if explist!=None else 'uvesexp.dat' if 'SQ12' in sample else 'hiresexp.dat'
    explist  = numpy.genfromtxt(explist,names=True,dtype=object,comments='!')
    settings = pathdata+'settings.dat' if settings==None and 'SQ12' in sample else settings
    settings = numpy.genfromtxt(settings,names=True,dtype=object,comments='!',skip_header=4)
    def uves_slope(cent,x,slope):
        return slope*(x-cent)
    def uves_whitmore(x,slope):
        if x < 3259.1:
            return slope*(3259.1-3900)
        if 3259.1 < x < 4518.8:
            return slope*(x-3900)
        if 4518.8 < x < 4726.9:
            y1 = -slope*3900 + slope*4518.8
            y2 = -slope*5800 + slope*4726.9
            a  = (y2-y1)/(4726.9-4518.8)
            b  = 4518.8 - y1/a
            return a*(x-b)
        if 4726.9 < x < 6834.9:
            return slope*(x-5800)
        if x > 6834.9:
            return slope*(6834.9-5800)
    def hires_amplitude(left,right,amplitude):
        middle = (left+right)/2
        x0,x1  = 3800.,5300.
        if middle<x0:
            return amplitude
        if x0<middle<x1:
            return amplitude * (numpy.cos((middle-x0)*numpy.pi/(x1-x0)))**2
        if x1<middle:
            return amplitude
    def hires_slope(x,slope):
        return slope*(x-5500.)
    
    middle = (left+right)/2
    shift = blflag = rdflag = 0
    # If data sample of the selected system is a UVES system
    if 'SQ12' in sample:
        # If Whitmore distortion model and non-zero slope
        if whitmore and slope!=0:
            # Calculate the shift based on the Whitmore model
            shift = uves_whitmore(middle,slope)
        # If selected system is using UVES SQUADER data
        else:
            # Search position of the quasar in the list of UVES exposures (explist)
            pos = numpy.where(explist['name']==qso.encode())[0][0]
            # Initialize total shift and exposure time parameter
            sumshift = sumcount = 0
            # Loop in the list of UVES exposures until break is called
            for l in range (pos,len(explist)):
                # If quasar name in list different from the selected system
                if explist['name'][l]!=qso.encode():
                    # Break the loop
                    break
                # For each exposure, loop over all the existing settings
                for k in range(len(settings)):
                    # Condition 1: Central wavelength of setting same than the exposure
                    cond1 = settings['cent'][k]==explist['cent'][l]
                    # Condition 2: Optical arm of setting same than the exposure
                    cond2 = settings['arm'][k]==explist['arm'][l]
                    # Condition 3: Optical mode of setting same than the exposure
                    cond3 = settings['mode'][k]==explist['mode'][l]
                    # If the 3 conditions are satisfied
                    if cond1==cond2==cond3==True:
                        # Define starting wavelength (in Angstrom) from value in the list of settings (in nm)
                        wbeg  = 10*float(settings['TS_min'][k])
                        # Define central wavelength (in Angstrom) from value in the list of settings (in nm)
                        cent  = 10*float(settings['cent'][k])
                        # Define ending wavelength (in Angstrom) from value in the list of settings (in nm)
                        wend  = 10*float(settings['TS_max'][k])
                        # Break the loop in the list of settings as the correct setting has been matched
                        break
                # If creating simulated spectrum and expind option used, define a random slope based on exposure settings
                if stdev!=0:
                    # If no slope already attributed or exposure not yet tabulated in shifts
                    if len(shifts)==0 or str(explist['dataset'][l]) not in shifts[:,0]:
                        # Determine slope for this exposure using gaussian distribution
                        slope = numpy.random.normal(slope,stdev)
                        # Store the exposure name and associated slope value calculated in shifts
                        shifts = numpy.vstack((shifts,[str(explist['dataset'][l]),str(slope)]))
                    # If exposure already tabulated in shifts
                    if len(shifts)>0 and str(explist['dataset'][l]) in shifts[:,0]:
                        # Find index position of this exposure in shifts
                        idx   = numpy.where(shifts[:,0]==explist['dataset'][l])[0][0]
                        # Extract previously calculated and stored slope value for this exposure
                        slope = float(shifts[idx,1])
                if explist['arm'][l]==b'BLUE' and wbeg < left and right < wend:
                    sumshift = sumshift + numpy.sqrt(float(explist['exptime'][l])) * uves_slope(cent,middle,slope)
                    sumcount = sumcount + numpy.sqrt(float(explist['exptime'][l]))
                    blflag = 1
                if explist['arm'][l]==b'RED' and wbeg < left and right < wend:
                    sumshift = sumshift + numpy.sqrt(float(explist['exptime'][l])) * uves_slope(cent,middle,slope)
                    sumcount = sumcount + numpy.sqrt(float(explist['exptime'][l]))
                    rdflag = 1
            shift = sumshift / sumcount
            #regtotal = regtotal + 1
            #if blflag==1 and rdflag==0:
            #    armflag = 'blue'
            #    regblue = regblue + 1
            #if blflag==0 and rdflag==1:
            #    armflag = 'red'
            #    regred = regred + 1
            #if blflag==rdflag==1:
            #    armflag = 'overlap'
            #    regboth = regboth + 1
    if ('CH01' or 'SA03') in sample:
        if slope==0:
            shift = 0
        elif whitmore and shape=='amplitude':
            shift = shift + hires_amplitude(left,right,slope)
        elif whitmore and shape=='slope':
            shift = shift + hires_slope(middle,slope)
        elif 'SA03' in sample:
            pos = numpy.where(numpy.logical_and(explist['name']==qso.encode(),explist['sample']==sample.encode()))[0][0]
            sumshift = sumcount = 0
            for l in range (pos,len(explist)):
                if explist['name'][l]!=qso.encode():
                    break
                wbeg = float(explist['blue'][l])
                cent = float(explist['cent'][l])
                wend = float(explist['red'][l])
                if wbeg < left and right < wend:
                    sumshift = sumshift + numpy.sqrt(float(explist['exptime'][l])) * uves_slope(cent,middle,slope)
                    sumcount = sumcount + numpy.sqrt(float(explist['exptime'][l]))
            shift = sumshift / sumcount
    return '{0:.4f}'.format(float(shift)/1000.)



import os,sys,numpy,shutil

def isfloat(value):
    """
    Check if string is float.
    """
    try:
      float(value)
      return True
    except ValueError:
      return False

def vpfit_setup(vpfit,instrument=None,chisq=1.E-5,no_alpha=False,atomdir=None,vpfsetup=None,copy=False):
    """
    VPFIT settings

    Note
    =====
    If running on the NCI/Raijin cluster, the absolute path is the following:
    `/home/561/vxd561/ASTRO/code/vpfit10/vpfit`
    """
    # Get full path to VPFIT executable
    vpfit = os.path.abspath(vpfit) if os.path.exists(vpfit) else shutil.which(vpfit)
    # Get path to atomic data file
    if atomdir==None:
        pathdata = os.path.dirname(os.path.realpath(__file__)) + '/data/'
        atomdir  = pathdata + 'atom.dat'
        atomdir  = pathdata + 'atom_new.dat'       if '--newatom'   in sys.argv else atomdir
        atomdir  = pathdata + 'atom_murphy.dat'    if '--murphy'    in sys.argv else atomdir
        atomdir  = pathdata + 'atom_mgisotope.dat' if '--mgisotope' in sys.argv else atomdir
    else:
        atomdir  = os.path.abspath(atomdir)
    # Store atomic data file into array
    atom = make_atom_list(atomdir)
    # Get path to VPFIT setting file
    if vpfsetup==None:
        vpfsetup = 'vp_setup_king.dat' if '9.5-king' in vpfit else 'vp_setup_hires.dat' if instrument=='hires' else 'vp_setup_uves.dat'
    else:
        vpfsetup = os.path.abspath(vpfsetup)
    # Store VPFIT settings into array
    vpsetup = numpy.loadtxt(vpfsetup,dtype=str,delimiter='\n',comments='!')
    for i,row in enumerate(vpsetup):
        if 'NOVARS' in row and no_alpha:
            vpsetup[i] = 'NOVARS 3'
    if '9.5-king' not in vpfit:
        vpsetup = numpy.hstack((vpsetup,['chisqthres %.E 2 %.E'%(chisq,chisq)]))
    return vpfit,atom,os.path.abspath(atomdir),vpsetup,os.path.abspath(vpfsetup)

def make_atom_list(atompath):
    """
    Store data from atom.dat
    """
    atom = numpy.empty((0,6))
    atomdat = numpy.loadtxt(atompath,dtype='str',delimiter='\n')
    for element in atomdat:
        l       = element.split()
        i       = 0      if len(l[0])>1 else 1
        species = l[0]   if len(l[0])>1 else l[0]+l[1]
        wave    = 0 if len(l)<i+2 else 0 if isfloat(l[i+1])==False else l[i+1]
        f       = 0 if len(l)<i+3 else 0 if isfloat(l[i+2])==False else l[i+2]
        gamma   = 0 if len(l)<i+4 else 0 if isfloat(l[i+3])==False else l[i+3]
        mass    = 0 if len(l)<i+5 else 0 if isfloat(l[i+4])==False else l[i+4]
        alpha   = 0 if len(l)<i+6 else 0 if isfloat(l[i+5])==False else l[i+5]
        if species not in ['>>','<<','<>','__']:
            atom = numpy.vstack((atom,[species,wave,f,gamma,mass,alpha]))
    return atom

def atomic_info(atom,atomid):
    """
    Find transition in atom list and extract information.

    Parameters
    ----------
    atomid : string
      Name of the transition, written as ion_wavelength.
    """
    pathdata = os.path.dirname(os.path.realpath(__file__)) + '/data/'
    target = [0,0,0,0,0]
    atomid = atomid.split('_')
    for i in range(len(atom)):
        element     = atom[i,0]
        wavelength  = atom[i,1]
        oscillator  = atom[i,2]
        gammavalue  = atom[i,3]
        qcoeff      = atom[i,5]
        if (len(atomid)>1 and element==atomid[0] and abs(float(wavelength)-float(atomid[1]))<abs(float(target[1])-float(atomid[1]))) \
           or (len(atomid)==1 and element==atomid[0]):
           target = [element,wavelength,oscillator,gammavalue,qcoeff] 
    if target==[0,0,0,0,0]:
        print(atomid,'not identifiable...')
        quit()
    return target

def vpfit_commands(nregs,fort='fort.13',action='d'):
    '''
    Generate list of commands to be run with VPFIT.

    Parameters
    ----------
    nregs : float
      Total number of regions.
    fort : str
      Input fort.13 file name
    action : str
      Action to be performed. Default is 'd' for display.
    '''
    opfile = open('fitcommands','w')
    opfile.write('%s\n'%action)         # Run display command + enter
    opfile.write('\n')                  # ...used default setup -> enter only
    opfile.write('\n')                  # Used default selfeter (logN) -> enter only
    opfile.write('%s\n'%fort)           # Insert fort file name + enter
    opfile.write('\n')                  # Plot the fitting region (default is yes) -> enter only
    if nregs>1:                         # If more than one transitions...
        opfile.write('\n')              # ...select first transition to start with (default)
    opfile.write('as\n\n\n')
    for line in range(nregs):
        opfile.write('\n\n\n\n')
    opfile.write('n\n\n')
    opfile.close()



import numpy,math

def dist2dip(qsoname):
    """
    Determine distance to dipole from specific quasar.

    Parameters
    ----------
    qsoname : string
      J2000 name of the quasar object
    
    Returns
    -------
    zem : float
      Emission redshift of the quasar
    ra : float
      Right Ascension
    dec : float
      Declinaison
    distance : float
      Distance to dipole from quasar object
    """
    idx = numpy.where(qsolist['name']==qsoname)[0][0]
    val = qsolist['RA\n(hh:mm:ss)'][idx].split(':')
    ra  = float(val[0])+float(val[1])/60+float(val[2])/3600         # in hours
    val = qsolist['DEC\n(dd:mm:ss)'][idx].split(':')
    if (val[0][0]=='-'):
        dec = float(val[0])-float(val[1])/60-float(val[2])/3600     # in degrees
    else:
        dec = float(val[0])+float(val[1])/60+float(val[2])/3600     # in degrees
    zem  = float(qsolist['z_em'][idx])
    # Dot product using spherical coordinates phi and theta in radians
    xdipole  = alphara*360/24.    # in degrees
    ydipole  = alphadec           # in degrees
    distance = spheredist(ra*360/24.,dec,xdipole,ydipole)
    return zem,ra,dec,distance

def spheredist(ra1,dec1,ra2,dec2):
    """
    Calculate spherical distance between 2 points.

    Parameters
    ----------
    ra1 : float
      Right ascension of the first object
    ded1 : float
      Declinaison of the first object
    ra2 : float
      Right ascension of the second object
    ded2 : float
      Declinaison of the seconds object

    Returns
    -------
    distance : float
      spherical distance between the 2 objects
    """
    theta1   = ra1*math.pi/180
    phi1     = math.pi/2-dec1*math.pi/180
    x1       = math.sin(phi1)*math.cos(theta1)
    y1       = math.sin(phi1)*math.sin(theta1)    
    z1       = math.cos(phi1)
    theta2   = ra2*math.pi/180
    phi2     = math.pi/2-dec2*math.pi/180
    x2       = math.sin(phi2)*math.cos(theta2)
    y2       = math.sin(phi2)*math.sin(theta2)    
    z2       = math.cos(phi2)
    distance = math.acos(x1*x2+y1*y2+z1*z2)
    distance = distance*180/math.pi
    return distance




import numpy
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit

def fit_parabola(x,y,xmin=None,xmax=None,ymin=None,ymax=None,plot_range=[None,None],show_text=True):
    """
    Fit parabola to the chi-square curves.
    """
    # Define plotting limits if not specified
    if plot_range==[None,None]: plot_range=[min(x),max(x)]
    # Determine in-window label position
    ymin = 0 if len(y)==0 else min(y)-1 if min(y)==max(y) else min(y) if ymin==None else ymin
    ymax = 1 if len(y)==0 else max(y)+1 if min(y)==max(y) else max(y) if ymax==None else ymax
    xpos = numpy.average(plot_range)
    ypos = ymax-0.05*(ymax-ymin)
    # Define fitting range
    imin = 0  if xmin==None else numpy.where(x>=xmin)[0][0]
    imax = -1 if xmax==None else numpy.where(x<=xmax)[0][-1]+1
    x,y  = x[imin:imax],y[imin:imax]
    x = numpy.array(x,dtype=float)
    y = numpy.array(y,dtype=float)
    # Execute parabolic fit
    A = numpy.vander(x,3)
    (coeffs, residuals, rank, sing_vals) = numpy.linalg.lstsq(A,y,rcond=None)
    f = numpy.poly1d(coeffs)
    # Define fitting variables
    xfit = numpy.arange(-100,100,0.0001)
    imid = abs(f(xfit)-min(f(xfit))).argmin()
    isig = abs(f(xfit)-(min(f(xfit))+1)).argmin()
    xmid = xfit[imid]
    xsig = abs(xfit[isig]-xfit[imid])
    xm1sig = xmid-xsig
    xp1sig = xmid+xsig
    plt.plot(xfit,f(xfit),c='red',lw=1)
    print('Slope: {0:>8}+/-{1:<8}'.format('%.4f'%xmid,'%.4f'%xsig))
    #print('Chisq: {0:>12}'.format(self.residuals))
    plt.axvline(x=xm1sig,ls='dotted',color='blue',lw=1)
    plt.axvline(x=xmid,ls='dashed',color='red',lw=1)
    plt.axvline(x=xp1sig,ls='dotted',color='blue',lw=1)
    if show_text:
        t1 = plt.text(xpos,ypos,r'$\chi^2_\mathrm{min}$ at %.4f $\pm$ %.4f'%(xmid,xsig),
                      color='red',fontsize=10,ha='center',va='top')
        t1.set_bbox(dict(color='white', alpha=0.7, edgecolor=None))
    return xmid,xsig,xm1sig,xp1sig

def fit_linear(x,y,yerr,slope,slope_error,xm1sig,xp1sig,xmin=None,xmax=None,ymin=None,ymax=None,plot_range=[None,None],show_text=True):
    """
    Do linear fit to da/a vs. distortion slope curves.
    """
    # Define plotting limits if not specified
    if plot_range==[None,None]: plot_range=[min(x),max(x)]
    # Determine in-window label position
    xpos = numpy.average(plot_range)
    ypos = ymax-0.05*(ymax-ymin)
    # Define fitting range
    imin = 0 if xmin==None else numpy.where(x>=xmin)[0][0]
    imax = -1 if xmax==None else numpy.where(x<=xmax)[0][-1]+1
    x,y,yerr = x[imin:imax],y[imin:imax],yerr[imin:imax]
    # Execute parabolic fit
    x = numpy.array(x,dtype=float)
    y = numpy.array(y,dtype=float)
    yerr = numpy.array(yerr,dtype=float)
    def func(func,a,b):
        return a + b*x
    pars,cov = curve_fit(func,x,y,sigma=yerr)
    # Define fitting variables
    xfit = numpy.arange(-100,100,0.001)
    yfit = pars[0] + pars[1]*xfit
    plt.plot(xfit,yfit,color='red',lw=1)
    imid = abs(xfit-slope).argmin()
    imin = abs(xfit-(slope-slope_error)).argmin()
    imax = abs(xfit-(slope+slope_error)).argmin()
    plt.axvline(x=xm1sig,ls='dotted',color='blue',lw=1)
    plt.axvline(x=xfit[imid],ls='dashed',color='red',lw=1)
    plt.axvline(x=xp1sig,ls='dotted',color='blue',lw=1)
    plt.axhline(y=yfit[imid],ls='dashed',color='red',lw=1)
    plt.axhline(y=yfit[imax],ls='dotted',color='blue',lw=1)
    plt.axhline(y=yfit[imin],ls='dotted',color='blue',lw=1)
    alpha      = yfit[imid]
    alpha_stat = numpy.average(yerr)
    alpha_syst = abs(yfit[imax]-yfit[imid])
    if show_text:
        t1 = plt.text(xpos,ypos,
                      r'$\Delta\alpha/\alpha$ = %.4f $\pm$ %.4f $\pm$ %.4f'%(yfit[imid],alpha_stat,alpha_syst),
                      color='red',fontsize=10,ha='center',va='top')
        t1.set_bbox(dict(color='white', alpha=0.7, edgecolor=None))
    print('Alpha: {0:>8}+/-{1:<6}+/-{2:<6}'.format('%.4f'%alpha,'%.4f'%alpha_stat,'%.4f'%alpha_syst))



import sys,os,numpy,matplotlib,seaborn
import matplotlib.pyplot as plt
from .utils import get_coddam,get_fit_name
from .forthandle import get_results

def curveplot(args):
    """
    Plot chi-square curves for real absorption systems.

    This operation will produce chi-square and da/a curves for every system
    that are selected. The slope range and other distortion details are
    retrieved from the distres table in CoDDAM so there is no need to define
    manually the parameters. The way the program works is to first loop over
    all the systems listed in CoDDAM and store the indexes of the targeted
    systems, then loop over each of those systems, retrieve the results and
    finally plot the results.

    Examples
    --------

    >>> alpha curve_real --instrument uves --chisq 1e-6 --mode single
    >>>                  --system J001602-001225/0.63630/UVES_squader
    """
    # Set up suitable lookup table
    database = getcoddam('distres',args.coddam)
    if args.instrument==None or args.mode==None:
        print('ERROR: Either --instrument or --mode is missing...')
        quit()
    # If curve not for specific system, save all results in file
    target = args.system
    if target==None:
        out = open('temp.dat','w')
    name = getfitname(args.instrument,args.mode,args.sample)
    # Identify indexes of systems from distortion table
    idxs = []
    for i in range(len(database)):
        cond0 = args.system in [None,database['system'][i]]
        cond1 = database['instrument'][i].lower()==args.instrument
        cond2 = database['whitmore'][i]=='yes' and '--whitmore' in sys.argv
        cond3 = database['whitmore'][i]=='no'  and '--whitmore' not in sys.argv
        cond4 = args.sample==None or args.sample in database['system'][i]
        if cond0 and cond1 and (cond2 or cond3) and cond4:
            idxs.append(i)
    # Initialize loop over all systems
    print('\nGet',args.mode,'distortion results for',len(idxs),'systems\n')
    systot = 0
    data  = numpy.empty((0,4))
    chisq = numpy.empty((0,7))
    model = numpy.empty((0,4))
    # Do loop over all targeted systems
    for i in idxs:
        # Define all specifications
        curvemode  = 'weight' if args.mode=='global' else 'individual'
        system     = database['system'][i]
        instrument = database['instrument'][i].lower()
        wminther   = float(database['thermal min_fit'][i])
        wmaxther   = float(database['thermal max_fit'][i])
        wminturb   = float(database['turbulent min_fit'][i])
        wmaxturb   = float(database['turbulent max_fit'][i])
        wminmom    = float(database['MoM min_fit'][i])
        wmaxmom    = float(database['MoM max_fit'][i])
        distmin    = float(database['min '+curvemode][i])
        distmax    = float(database['max '+curvemode][i])
        distsep    = float(database['step '+curvemode][i])
        model      = 'model-%.1f'%float(database['model '+curvemode][i])
        test       = 'v10_chisq%.E'%float(database['threshold '+curvemode][i])
        test       = test + '_all' if instrument=='uves' else test + '_slope'
        test       = test + '_step%.E'%distsep
        test       = test + '_prev' if '--previous' in sys.argv else test
        test       = test + '_jw' if '--whitmore' in sys.argv else test
        if distmin < distmax <= 0:
            distlist = numpy.arange(distmax,distmin-0.001,-distsep)
            distplot = sorted(distlist)
        elif distmax > distmin >= 0:
            distlist = numpy.arange(distmin,distmax+0.001,distsep)
            distplot = sorted(distlist)
        elif distmin < 0 < distmax:
            distlist = numpy.hstack((numpy.arange(0,distmax+0.001,distsep),numpy.arange(0,distmin-0.001,-distsep)))
            distplot = sorted(numpy.delete(distlist,0))
        fitres = getchisquare(args.path,system,model,test,distplot)
        # If curves to be produced for individual systems, plot curve and save results
        if args.mode=='single':
            print('\n{0:<10}\n{1:<7} | {2:<8}\n'.format(system,model,test))
            if target==None:
                out.write('{:<40}\t'.format(system))
                if len(fitres)!=len(distplot):
                    out.write('{:>10}\t'.format('-')*15+'\n')
            #if len(fitres)==len(distplot):
            loop = True if target==None else False
            curves(system,fitres,distmin,distmax,fit=args.fit,loop=loop)
        # If curve to be produced as a global weighted mean
        if args.mode=='global':
            cond1  = len(fitres)==len(distplot)
            cond2  = database['parabola weight'][i]=='good'
            if cond1 and cond2:
                systot += 1
                if len(chisq)==0:
                    chisq = fitres[:,[0,1,2,5,6,9,10]]
                else:
                    chisq[:,1:] += fitres[:,[1,2,5,6,9,10]]
            therslope = '-' if len(fitres[:,2])==0  else '%.2f'%numpy.mean(fitres[:,2])
            turbslope = '-' if len(fitres[:,6])==0  else '%.2f'%numpy.mean(fitres[:,6])
            momslope  = '-' if len(fitres[:,10])==0 else '%.2f'%numpy.mean(fitres[:,10])
            print('{0:>10}\t{1:>10}\t{2:>10}'.format(therslope,turbslope,momslope))
            curves(system,fitres,distmin,distmax,args.fit,loop)
        if args.mode=='model' and len(fitres)==len(distplot):
            for j in range(3):
                x    = fitres[:,0]
                y    = fitres[:,2+4*j]
                xfit = numpy.arange(-5,5,0.0001)
                yfit = fitparabola2(x,y,fitrange=xfit)
                if len(model)==0:
                    model = numpy.zeros((len(xfit),4),dtype=float)
                    model[:,0] = xfit
                model[:,j+1] += yfit
        if args.mode=='alpha' and len(fitres)>0:
            data = numpy.vstack((data,[database['redshift'][i],
                                       database['MoM alpha'][i],
                                       database['MoM error'][i],
                                       database['name'][i]]))
            if '--show' in sys.argv:
                print('{:>15}'.format(database['redshift'][i]),)
                print('{:>15}'.format(database['MoM alpha'][i]),)
                print('{:>15}'.format(database['MoM alpha'][i]),)
                print('{:>15}'.format(database['name'][i]))
    if target==None:
        out.close()
        os.system('mv temp.dat %s/results/curves/%s/%s.dat'%(home,instrument,name))
    if args.mode in ['global','model']:
        print('Total number of systems used:',systot)
        results = chisq if mode=='global' else model
        weight(args.mode)
    if args.mode=='alpha':
        print(data)

def getchisquare(fitdir,system,fitmodel,test,distplot,model=None,predicted=None):
    """
    Extract chi-square from results
    """
    if '--show' in sys.argv:
        print('\n{0:>7}{1:>12}{2:>12}{3:>12}{4:>12}{5:>12}{6:>12}{7:>12}{8:>12}{9:>12}{10:>12}{11:>12}{12:>12}\n'.format('dist','ther_ndf','therchisq','theralpha','thererror','turb_ndf','turbchisq','turbalpha','turberror','mom_ndf','momchisq','momalpha','momerror'))
    fitres   = numpy.empty((0,13))
    for i in distplot:
        strslope = '0.000' if round(i,3)==0 else str('%.3f'%i).replace('-','m') if '-' in str(i) else 'p'+str('%.3f'%i)
        distpath = fitdir+'/'+system+'/'+fitmodel+'/runs/'+test+'/'+strslope+'/'
        therfort = distpath+'/thermal/thermal.18'
        turbfort = distpath+'/turbulent/turbulent.18'
        quasar   = system.split('/')[0]
        zabs     = system.split('/')[1]
        sample   = system.split('/')[2]
        if os.path.exists(distpath):
            ther,turb,mom = getresults(therfort,turbfort)
            dist          = round(i,3)
            fitres        = numpy.vstack((fitres,[None]*13))
            fitres[-1,0]  = dist
            fitres[-1,1]  = ther.df
            fitres[-1,2]  = ther.chisq
            fitres[-1,3]  = ther.alpha
            fitres[-1,4]  = ther.error
            fitres[-1,5]  = turb.df
            fitres[-1,6]  = turb.chisq
            fitres[-1,7]  = turb.alpha
            fitres[-1,8]  = turb.error
            fitres[-1,9]  = mom.df
            fitres[-1,10] = mom.chisq
            fitres[-1,11] = mom.alpha
            fitres[-1,12] = mom.error
            if '-' not in fitres[-1]:
                if '--show' in sys.argv:
                    print('{:>7} '.format('%.3f'%dist),)
                    print('{:>10}'.format('%i'%ther.df),)
                    print('{:>11}'.format('%.4f'%ther.chisq),)
                    print('{:>11}'.format('%.4f'%ther.alpha),)
                    print('{:>11}'.format('%.4f'%ther.error),)
                    print('{:>11}'.format('%i'%turb.df),)
                    print('{:>11}'.format('%.4f'%turb.chisq),)
                    print('{:>11}'.format('%.4f'%turb.alpha),)
                    print('{:>11}'.format('%.4f'%turb.error),)
                    print('{:>11}'.format('%i'%mom.df),)
                    print('{:>11}'.format('%.4f'%mom.chisq),)
                    print('{:>11}'.format('%.4f'%mom.alpha),)
                    print('{:>11}'.format('%.4f'%mom.error))
                chisq = turb.chisq if model=='turb' else ther.chisq if model=='ther' else mom.chisq
                ndf   = turb.df    if model=='turb' else ther.df    if model=='ther' else mom.df
                alpha = turb.alpha if model=='turb' else ther.alpha if model=='ther' else mom.alpha
                error = turb.error if model=='turb' else ther.error if model=='ther' else mom.error
            else:
                fitres = numpy.delete(fitres,-1,0)
    if '--show' in sys.argv:
        print('\n')
    dist,x,yabs,y = [],[],[],[]
    if len(fitres)>0:
        dist = [round(i,3) for i in fitres[:,0]]
        yndf = fitres[:,1]
        yabs = fitres[:,2]
        yred = yabs/yndf
        #if predicted=='null':
        #    A = numpy.vander(dist,3)
        #    (coeffs, residuals, rank, sing_vals) = numpy.linalg.lstsq(A,yalp)
        #    f = numpy.poly1d(coeffs)
        #    x = numpy.arange(-0.5,0.5,0.0001)
        #    imid = abs(f(x)-min(f(x))).argmin()
        #    imid = abs(dist-x[imid]).argmin()
        #    minchisq = yalp[imid]
        #    y = [numpy.exp(-minchisq/2)*numpy.exp(-chisq/2.) for chisq in yalp]
        #    y = yalp
        #else:
        #    A = numpy.vander(dist,3)
        #    (coeffs, residuals, rank, sing_vals) = numpy.linalg.lstsq(A,yabs)
        #    f = numpy.poly1d(coeffs)
        #    x = numpy.arange(-0.5,0.5,0.0001)
        #    imid = abs(f(x)-min(f(x))).argmin()
        #    imid = abs(dist-x[imid]).argmin()     
        #    minchisq = yred[imid]
        #    #norm = 10**int(math.log10(numpy.exp(-minchisq/2)*numpy.exp(-minchisq/2.)))
        #    y = [numpy.exp(-minchisq/2)*numpy.exp(-chisq/2.) for chisq in yred]
        #    #y = yred
    dist     = dist
    yabs     = yabs
    fitres   = fitres
    return fitres
    
def curves(system,fitres,distmin,distmax,fit,loop=False):
    """
    Do scatter plot of both chi-square and da/a results versus distortion slope
    """
    degensys = ['J111113-080402/3.60770/UVES_squader',
                'J034943-381031/3.02470/HIRES_prochaska']
    # Initializing the figure        
    matplotlib.rcParams.update(matplotlib.rcParamsDefault)
    plt.rc('font', size=2, family='sans-serif')
    plt.rc('axes', labelsize=10, linewidth=0.2)
    plt.rc('legend', fontsize=10, handlelength=10)
    plt.rc('xtick', labelsize=7)
    plt.rc('ytick', labelsize=7)
    plt.rc('lines', lw=0.2, mew=0.2)
    plt.rc('grid', linewidth=0.5)
    plt.style.use('seaborn')
    fig = plt.figure(figsize=(8,5))
    plt.subplots_adjust(left=0.05, right=0.97, bottom=0.05, top=0.95, hspace=0.2, wspace=0.2)
    model = ['Thermal','Turbulent','MoM']
    for j in range(len(model)):
        if loop:
            out.write('{0:>10}\t'.format('%.2f'%numpy.mean(fitres[:,2+4*j])))
        x     = fitres[:,0]
        y     = fitres[:,2+4*j]
        label = model[j]+' $\chi^2$'
        ax    = plt.subplot(2,3,1+j,xlim=[distmin,distmax])
        ax.errorbar(x,y,fmt='o',ms=4,markeredgecolor='none',ecolor='grey',alpha=0.7,color='black')
        ymin  = 0 if len(y)==0 else min(y)-1 if min(y)==max(y) else min(y)
        ymax  = 1 if len(y)==0 else max(y)+1 if min(y)==max(y) else max(y)
        if system not in degensys and fit in ['chisq','both']:
            fitparabola2(x,y,model=model[j])
        ax.set_ylim(ymin,ymax)
        y_formatter = matplotlib.ticker.ScalarFormatter(useOffset=False)
        ax.yaxis.set_major_formatter(y_formatter)
        t1 = ax.text((distmin+distmax)/2,ymax-0.1*(ymax-ymin),system,color='grey',ha='center',fontsize=6)
        t1.set_bbox(dict(color='white', alpha=0.7, edgecolor=None))
        ax.set_title(label,fontsize=10)
        if loop:
            slope = 'badwhit' if system in badwhit and '--whitmore' in sys.argv else \
                    'degenerate' if system in degensys else '%.4f'%slope
            error = 'badwhit' if system in badwhit and '--whitmore' in sys.argv else \
                    'degenerate' if system in degensys else '%.4f'%slope_error
            out.write('{0:>10}\t{1:>10}\t'.format(slope,error))
        x     = fitres[:,0]
        y     = fitres[:,3+4*j]
        yerr  = fitres[:,4+4*j]
        ymin  = 0 if len(y)==0 else min(y)-1 if min(y)==max(y) else min(y-yerr)
        ymax  = 1 if len(y)==0 else max(y)+1 if min(y)==max(y) else max(y+yerr)
        label = model[j]+r' $\Delta\alpha/\alpha$ $(10^{-5})$'
        ax    = plt.subplot(2,3,4+j,xlim=[distmin,distmax],ylim=[ymin,ymax])
        ax.errorbar(x,y,yerr=yerr,fmt='o',ms=4,markeredgecolor='none',ecolor='grey',alpha=0.7,color='black',lw=0.5)
        if system not in degensys and fit in ['alpha','both']:
            fitlinear(x,y,yerr,model=model[j])
        y_formatter = matplotlib.ticker.ScalarFormatter(useOffset=False)
        ax.yaxis.set_major_formatter(y_formatter)
        t1 = ax.text((distmin+distmax)/2,ymax-0.1*(ymax-ymin),system,color='grey',ha='center',fontsize=6)
        t1.set_bbox(dict(color='white', alpha=0.7, edgecolor=None))
        ax.set_title(label,fontsize=10)
        if loop:
            if system in badwhit and '--whitmore' in sys.argv:
                alpha = error = syst = 'badwhit'
            elif system in degensys:
                alpha = error = syst = 'degenerate'
            else:
                alpha = '%.4f'%alpha
                error = '%.4f'%alpha_stat
                syst  = '%.4f'%alpha_syst
            out.write('{0:>10}\t{1:>10}\t{2:>10}\t'.format(alpha,error,syst))
    if loop==True:
        out.write('\n')
    plt.savefig('chisq.pdf')
    plt.close(fig)

def weight(self):

    plt.rc('font', size=5, family='sans-serif')
    plt.rc('axes', labelsize=10, linewidth=0.2)
    plt.rc('legend', fontsize=10, handlelength=10)
    plt.rc('xtick', labelsize=10)
    plt.rc('ytick', labelsize=10)
    plt.rc('lines', lw=0.2, mew=0.2)
    plt.rc('grid', linewidth=0.2)
    fig = plt.figure(figsize=(5,8))
    plt.subplots_adjust(left=0.15, right=0.95, bottom=0.07, top=0.95, hspace=0.3, wspace=0)
    model = ['Thermal','Turbulent','MoM']
    for i in range(len(model)):
        if mode=='model':
            x      = results[:,0]
            y      = results[:,1+i]
            xmin   = -0.1
            xmax   = 0
            imin   = abs(x-xmin).argmin()
            imax   = abs(x-xmax).argmin()
            ymin   = min(y[imin:imax])
            ymax   = max(y[imin:imax])
            imid   = abs(y-min(y)).argmin()
            isig   = abs(y-(min(y)+1)).argmin()
            xmid   = x[imid]
            xsig   = abs(x[isig]-x[imid])
            xm1sig = xmid-xsig
            xp1sig = xmid+xsig
            print('Slope: {0:>8}+/-{1:<8}'.format('%.4f'%xmid,'%.4f'%xsig),)
            ax = plt.subplot(3,1,i+1)
            ax.plot(x,y,color='black',alpha=0.7,lw=2)
            axvline(x=xm1sig,ls='dotted',color='red',lw=1)
            axvline(x=xmid,ls='dashed',color='red',lw=1)
            axvline(x=xp1sig,ls='dotted',color='red',lw=1)
            t1 = text(0,ymax-0.17*(ymax-ymin),'$1\sigma$ : %.4f+/-%.4f'%(xmid,xsig),color='red',fontsize=6,ha='center')
            t1.set_bbox(dict(color='white', alpha=0.7, edgecolor=None))
        if mode=='global':
            x    = results[:,0]
            y    = results[:,2*(i+1)]
            xmin = distmin
            xmax = distmax
            ymin = min(y)
            ymax = max(y)
            #imin = abs(x+0.03).argmin()
            #imax = abs(x+0.14).argmin()
            A = numpy.vander(x[8:-1],3)
            (coeffs, residuals, rank, sing_vals) = numpy.linalg.lstsq(A,y[8:-1])
            f      = numpy.poly1d(coeffs)
            xfit   = numpy.arange(-50,50,0.0001)
            imid   = abs(f(xfit)-min(f(xfit))).argmin()
            isig   = abs(f(xfit)-(min(f(xfit))+1)).argmin()
            xmid   = xfit[imid]
            xsig   = abs(xfit[isig]-xfit[imid])
            xm1sig = xmid-xsig
            xp1sig = xmid+xsig
            print('Slope: {0:>8}+/-{1:<8}'.format('%.4f'%xmid,'%.4f'%xsig),)
            ax = plt.subplot(3,1,i+1)
            ax.scatter(x,y,color='black',alpha=0.7,lw=2)
            ax.plot(xfit,f(xfit),c='red',lw=1)
            axvline(x=xm1sig,ls='dotted',color='red',lw=1)
            axvline(x=xmid,ls='dashed',color='red',lw=1)
            axvline(x=xp1sig,ls='dotted',color='red',lw=1)
            t1 = text((xmin+xmax)/2,ymax-0.17*(ymax-ymin),'$1\sigma$ : %.4f+/-%.4f'%(xmid,xsig),color='red',fontsize=10,ha='center')
            t1.set_bbox(dict(color='white', alpha=0.7, edgecolor=None))
        if i==0 and '--whitmore' in sys.argv:
            plt.title('Simplistic model',fontsize=10)
        elif i==0:
            plt.title('Our model',fontsize=10)
        ax.set_xlim(xmin,xmax)
        ax.set_ylim(ymin,ymax)
        ax.axvline(x=0,color='black')
        #ax.set_title(model[i],color='red',fontsize=10)
        ax.set_xlabel('Applied distortion slope (in m/s/$\AA$)',fontsize=10)
        ax.set_ylabel(model[i]+' global absolute $\chi^2$',fontsize=10)
        ax.yaxis.set_major_formatter(matplotlib.ticker.ScalarFormatter(useOffset=False))
    plt.savefig(name+'.pdf')
    plt.clf()



import sys,os,re,numpy,operator
import matplotlib.pyplot as plt
import matplotlib as mpl
from .utils import get_coddam
from .vpfit import atomic_info
from .constants import *

def getinfo(instrument,fitdir):
    """
    Store information regarding metals in multiple arrays.

    Notes
    -----
    This method doesn't take or return any arguments. Instead, the
    relevant information is extracted from and inserted in the setup
    object initialised in the settings method.

    The 4 data arrays that are stored in the setup object are the
    followings: infoatom, which store unique atomic data from all the
    transitions present in the selected sample; infosys, which contains
    the store the contrast for each system; infoleg, which store the
    minimum and maximum q-coefficient for each contrast, and infometal,
    which store all the atomic data for each system.
    """
    publist   = getcoddam('publist')
    curlist   = getcoddam('curlist')
    infoatom  = numpy.empty((0,6),dtype=object)  # List of all transitions used in sample [atomID,wavelength]
    infosys   = numpy.empty((0,4),dtype=object)  # List of absorption redshift and q variability [zabs,qvar]
    infoleg   = numpy.empty((0,3),dtype=object)  # List for legend in qvariability plot [qvar,qmin,qmax]
    infometal = numpy.empty((0,7),dtype=object)  # List of all metals used in sample [args.atom[index,:]+[zabs]]
    for system in publist['system']:
        qso    = system.split('/')[0]
        zabs   = float(system.split('/')[1])
        sample = system.split('/')[2]
        idx    = numpy.where(curlist['system']==system)[0][0]
        model  = curlist['jak12'][idx]
        if os.path.exists('%s/%s/model-%s/model/header.dat'%(fitdir,system,model))==False:
            pass
        elif (instrument==None or instrument.upper() in sample) and system not in env('outliers'):
            infohead = numpy.empty((0,7),dtype=object)
            line13 = numpy.loadtxt('%s/%s/model-%s/model/header.dat'%(fitdir,system,model),
                                   dtype='str',delimiter='\n')
            for k in range (len(line13)):
                if line13[k].split()[0][0]!='!' and 'external' not in line13[k].split():
                    atomID    = line13[k].split()[0]
                    component = atominfo(atomID)
                    infohead  = numpy.vstack((infohead ,component+[zabs,atomID]))
                    infometal = numpy.vstack((infometal,component+[zabs,atomID]))
                if atomID not in infoatom[:,-1]:
                    infoatom  = numpy.vstack((infoatom,component+[atomID]))
            qmin = min([float(k) for k in infohead[:,-3]])
            qmax = max([float(k) for k in infohead[:,-3]])
            qvar = abs(qmax-qmin)
            imin = numpy.where(infohead[:,-3]=='%i'%qmin)[0][0]
            imax = numpy.where(infohead[:,-3]=='%i'%qmax)[0][0]
            qmin = '%s%i'%(infohead[imin,0],float(infohead[imin,1]))
            qmax = '%s%i'%(infohead[imax,0],float(infohead[imax,1]))
            if qvar not in infoleg[:,0]:
                info = numpy.array([qvar,qmin,qmax],dtype=object)
                infoleg = numpy.vstack((infoleg,info))
            sys     = numpy.array([zabs,qvar,'',sample],dtype=object)
            infosys = numpy.vstack((infosys,sys))
    return numpy.array(sorted(infoatom,key=operator.itemgetter(1))),infosys,\
        numpy.array(sorted(infoleg,key=operator.itemgetter(0),reverse=True)),infometal

def plot_sample():
    """
    Plot redshift vs. rest-wavelength for every transitions
    """
    getinfo(instrument,fitdir)
    os.chdir(args.home+'/results/metals/')
    qcoeff  = numpy.array(args.infometal[:,-3],dtype=float)
    lamrest = numpy.array(args.infometal[:, 1],dtype=float)
    redabs  = numpy.array(args.infometal[:,-2],dtype=float)
    obswave = lamrest*(redabs+1)
    print('min / max observed wavelength: %.2f / %.2f'%(min(obswave),max(obswave)))
    fig = figure(figsize=(11.69,8.27))
    plt.subplots_adjust(left=0.07, right=0.87, bottom=0.1, top=0.95, hspace=0, wspace=0)
    ax = subplot(111)
    xlim(1300,3500)
    ylim(0,max(redabs)+1)
    ypos = max(redabs)+0.6
    for i in range (len(args.infoatom)):
        wave  = float(args.infoatom[i,1])
        trans = args.infoatom[i,-1].replace('_',' ')
        vlines(x=wave,ymin=-1,ymax=ypos,color='black',linestyles='dotted',zorder=3,lw=0.1)
        text(wave,ypos,trans,ha='left',fontsize=5)
        ypos -= 0.05
    scatter(lamrest,redabs,marker='o',s=10,edgecolors='none',zorder=3,\
            c=qcoeff,cmap=mpl.cm.cool,vmin=min(qcoeff),vmax=max(qcoeff))
    xlabel('Transition rest-frame wavelength',fontsize=12)
    ylabel('Redshift',fontsize=12)
    ax1  = fig.add_axes([0.87,0.1,0.04,0.85])
    cmap = mpl.cm.cool
    norm = mpl.colors.Normalize(vmin=min(qcoeff),vmax=max(qcoeff))
    cb1  = mpl.colorbar.ColorbarBase(ax1,cmap=cmap,norm=norm)
    cb1.set_label('q-coefficient',fontsize=12)
    name = 'metals_murphy' if instrument=='hires' else 'metals_king'
    savefig(name+'.pdf') if args.tbs else plt.show()
    plt.close(fig)

def qlist(instrument=None,fitdir='/Users/vincent/Documents/systems/',save_name=None):
    """
    Plot q-coefficient vs. rest-wavelength and frequency
    """
    infoatom,infosys,infoleg,infometal = getinfo(instrument,fitdir)
    qarray = numpy.empty((0,5),dtype=object)
    for i in range(len(infometal)):
        trans = infometal[i,-1]
        lrest = infometal[i, 1]
        qcoef = infometal[i,-3]
        zabs  = infometal[i,-2]
        if trans not in qarray[:,0]:
            qarray = numpy.vstack((qarray,[trans,lrest,qcoef,float(zabs),1]))
        else:
            idx = numpy.where(qarray[:,0]==trans)[0][0]
            qarray[idx,-2] = float(qarray[idx,-2]) + float(zabs)
            qarray[idx,-1] = int(qarray[idx,-1]) + 1
    x = numpy.array([i for i in qarray[:,1]],dtype=float)
    y = numpy.array([i for i in qarray[:,2]],dtype=float)
    n = numpy.array([i for i in qarray[:,4]],dtype=float)
    z = numpy.array([float(trans[3])/float(trans[4]) for trans in qarray],dtype=float)
    fig = plt.figure(figsize=(12,8))
    plt.style.use('seaborn')
    plt.subplots_adjust(left=0.08, right=0.88, bottom=0.1, top=0.87, hspace=0, wspace=0)
    ax1 = plt.subplot(111,xlim=[1300,3000],ylim=[-2000,3000])
    ax1.scatter(x,y,marker='o',s=40*n,edgecolors='none',alpha=0.6,c=z,
                cmap=mpl.cm.rainbow,vmin=min(z),vmax=max(z),zorder=2)
    plt.xlabel('rest-wavelength',fontsize=15)
    plt.ylabel('q-coefficient',fontsize=15)
    l1 = plt.scatter([],[], s=40*1, edgecolors='none')
    l2 = plt.scatter([],[], s=40*10, edgecolors='none')
    l3 = plt.scatter([],[], s=40*50, edgecolors='none')
    l4 = plt.scatter([],[], s=40*100, edgecolors='none')
    leg = plt.legend([l1,l2,l3,l4],["1","10","50","100"],ncol=4,frameon=False,fontsize=12,
                     handlelength=3, loc=8, borderpad = 0, fancybox=False,
                     bbox_to_anchor=[0.5,1.06],columnspacing=5,
                     handletextpad=1.5, scatterpoints = 1)
    for point in leg.legendHandles:
        point.set_color('#eaeaf2')
    ax1  = fig.add_axes([0.88,0.1,0.04,0.77])
    cmap = mpl.cm.rainbow
    vmin = 1.0 #min(z)
    vmax = 2.6 #max(z)
    norm = mpl.colors.Normalize(vmin=vmin,vmax=vmax)
    cb1  = mpl.colorbar.ColorbarBase(ax1,cmap=cmap,norm=norm)
    cb1.set_label('redshift',fontsize=15)
    name = 'qlist_murphy' if instrument=='hires' else 'qlist_king' if instrument=='uves' else 'qlist_all'
    plt.show() if save_name==None else plt.savefig(save_name)
    plt.close(fig)

def qvar():
    """
    Plot contrast against redshift for every absorber
    """
    getinfo(instrument,fitdir)
    os.chdir(args.home+'/results/metals/')
    fig = figure(figsize=(12,8))
    plt.subplots_adjust(left=0.07, right=0.95, bottom=0.1, top=0.95, hspace=0, wspace=0)
    xmax = 4.3
    ymax = max(args.infosys[:,1])+50
    qcoeff = args.infosys[:,1]
    ax1 = subplot(111,xlim=[0,xmax],ylim=[0,ymax])
    ax1.scatter(args.infosys[:,0],args.infosys[:,1],marker='o',s=50,edgecolors='none',
                alpha=0.8,c=qcoeff,cmap=mpl.cm.cool,vmin=min(qcoeff),vmax=max(qcoeff),zorder=2)
    ax1.yaxis.set_major_locator(plt.FixedLocator(numpy.arange(0,ymax,500)))
    xlabel('Redshift',fontsize=10)
    ylabel('q variability',fontsize=10)
    print(str(len(args.infosys)),'absorbers')
    for i in range (len(args.infoleg)):
        num  = len(numpy.where(args.infosys[:,1]==args.infoleg[i,0])[0])
        print('{:>5} : {:>10} - {:<10} : {:<5}'.format('%i'%args.infoleg[i,0],
                                                       args.infoleg[i,1],
                                                       args.infoleg[i,2],'%i'%num))
    divider = make_axes_locatable(ax1)
    ax2 = divider.append_axes("top",1.2,xlim=[0,xmax],pad='2%')
    ax2.hist(args.infosys[:,0],bins=100,histtype='stepfilled',color='r',alpha=0.5,lw=0.2)
    setp(ax2.get_xticklabels(), visible=False)
    name = 'qvar_murphy' if instrument=='hires' else 'qvar_king' if instrument=='uves' else 'qvar_all'
    savefig(name+'.pdf') if args.tbs else plt.show()
    plt.close(fig)
    
def mgfe(fitdir='/Users/vincent/Documents/systems/',save_name=None):
    '''
    Distribution of FeII and MgII transitions.
    '''
    publist = getcoddam('publist')
    plt.rc('font', size=2, family='sans-serif')
    plt.rc('axes', labelsize=8, linewidth=1)
    plt.rc('legend', fontsize=2, handlelength=10)
    plt.rc('xtick', labelsize=10)
    plt.rc('ytick', labelsize=10)
    plt.rc('lines', lw=1, mew=0.2)
    plt.rc('grid', linewidth=1)
    # List absorption systems to study
    mgii_ther = []
    mgii_turb = []
    feii_ther = []
    feii_turb = []
    for system in publist['system']:
        test = 'v10_chisq1E-06' if 'HIRES' in system else 'v10_chisq1E-06_all'
        distpath = fitdir+'/'+system+'/model-2.1/runs/'+test+'/0.000/'
        if system not in env('outliers') and os.path.exists(distpath+'thermal/thermal_fit.13'):
            therfortloc = distpath+'/thermal/thermal_fit.13'
            if os.path.exists(therfortloc)==True:
                i,flag  = 0,0
                fort = [line.replace('\n','') for line in open(therfortloc,'r')]
                while i < len(fort):
                    if '*' in fort[i]:
                        i,flag = i+1,flag+1
                    if flag==2:
                        coldens = re.compile(r'[^\d.-]+').sub('',fort[i].split()[1])
                        if fort[i].split()[0]=='MgII':
                            mgii_ther.append(float(coldens))
                        if fort[i].split()[0]=='FeII':
                            feii_ther.append(float(coldens))
                    i += 1
            turbfortloc = distpath+'/turbulent/turbulent_fit.13'
            if os.path.exists(turbfortloc)==True:
                i,flag  = 0,0
                fort = [line.replace('\n','') for line in open(turbfortloc,'r')]
                while i < len(fort):
                    if '*' in fort[i]:
                        i,flag = i+1,flag+1
                    if flag==2:
                        coldens = re.compile(r'[^\d.-]+').sub('',fort[i].split()[1])
                        if fort[i].split()[0]=='MgII':
                            mgii_turb.append(float(coldens))
                        if fort[i].split()[0]=='FeII':
                            feii_turb.append(float(coldens))
                    i += 1
    fig = plt.figure(figsize=(12,8),dpi=150)
    plt.style.use('seaborn')
    plt.subplots_adjust(left=0.05, right=0.95, bottom=0.05, top=0.95, hspace=0.15, wspace=0.2)
    ax = plt.subplot(2,2,1)
    n, bins, patches = ax.hist(feii_turb, 20, histtype='step', stacked=True, fill=True,alpha=0.5)
    ax.set_title('FeII turbulent',fontsize=10)
    ax = plt.subplot(2,2,2)
    n, bins, patches = ax.hist(feii_ther, 20, histtype='step', stacked=True, fill=True,alpha=0.5)
    ax.set_title('FeII thermal',fontsize=10)
    ax = plt.subplot(2,2,3)
    n, bins, patches = ax.hist(mgii_turb, 20, histtype='step', stacked=True, fill=True,alpha=0.5)
    ax.set_title('MgII turbulent',fontsize=10)
    ax = plt.subplot(2,2,4)
    n, bins, patches = ax.hist(mgii_ther, 20, histtype='step', stacked=True, fill=True,alpha=0.5)
    ax.set_title('MgII thermal',fontsize=10)
    plt.show() if save_name==None else plt.savefig(save_name)
    plt.close(fig)



import os,numpy,re
from .utils import read_spec,check_file_path
from .forthandle import create_fit13
from .vpfit import vpfit_commands,vpfit_setup
from .distortion import getshift
from .constants import *

def create_simulated_model(syspath,vpfit,instrument,qso,sample,slope,snr,explist=None,settings=None,shape='slope',chisq=1.E-5,atomdir=None,vpfsetup=None,expind=False):

    # Check path to absorption system repository is different where the simulated model is created
    assert os.path.abspath(syspath)!=os.getcwd(), '\n\nAbsorption system main directory (--syspath) cannot be your current location. Either correct the syspath variable or run the program from a different location (sub-folder are allowed). Abort.\n'
    
    # Get absolute path for further calls
    explist,settings,atomdir,vpfsetup = check_file_path([explist,settings,atomdir,vpfsetup])

    # Extract VPFIT information
    vpfit,atom,atomdir,vpsetup,vpfsetup = vpfit_setup(vpfit,instrument,no_alpha=True,copy=True,atomdir=atomdir,vpfsetup=vpfsetup)
        
    # Copy transition list, fort.18 and fort.26 files in model folder
    syspath = os.path.abspath(syspath)
    os.system('mkdir -p model/original model/data')
    os.system('cp %s/model/header.dat model/'%syspath)
    os.system('cp %s/model/turbulent.18 model/fort.18'%syspath)
    os.system('cp %s/model/turbulent.26 model/fort.26'%syspath)
    os.chdir('model')
    
    # Create fort_fit.13 from the fort.26 file and store contents in fort13 array
    create_fit13('fort',simulation=True)
    fort13 = open('fort_fit.13','r')
    fort13 = [line.strip() for line in fort13]
    os.system('rm fort*')
    
    # Initialise output files
    file1  = open('original/fort.13','w')
    file2  = open('turbulent.13','w')
    regs   = numpy.empty((0,2))
    
    # Loop over all the components and prepare ouput model
    i,flag,ihead,idx = 0,0,0,1
    while i<len(fort13):
        line = fort13[i]
        # End loop when empty line is reached
        if len(line.strip())==0:
            break
        elif '*' in line:
            file1.write(line+'\n')
            file2.write(line+'\n')
            flag += 1
        elif flag==1:
            spec = line.split()[0].split('/')[-1]
            regs = numpy.vstack((regs,[float(line.split()[2]),float(line.split()[3])]))
            head = line.replace(line.split()[0],'data/'+spec)
            file1.write(head+'\n')
            file2.write(line.replace(line.split()[0],'data/spec%02i.txt'%(idx))+'\n')
            idx += 1
            ihead += 1
        elif flag==2:
            val = line.split()
            if len(val[0])==1:
                vals = [val[0]+' '+val[1],val[2],val[3],val[4],val[5],val[6],val[7],val[8]]
            else:
                vals = [val[0],val[1],val[2],val[3],val[4],val[5],val[6],val[7]]
            comp = numpy.delete(vals,4,0)
            file1.write('   {0:<6} {1:>10} {2:>15} {3:>11} {4:>10} {5:>10} {6:>3}\n'.format(*comp))
            vals[4] = '0.000'+" ".join(re.findall("[a-zA-Z]+",vals[4][-2:]))
            file2.write('   {0:<6} {1:>10} {2:>15} {3:>11} {4:>11} {5:>10} {6:>10} {7:>3}\n'.format(*vals))
        i += 1
    file1.close()
    file2.close()
    
    # Go to relevant folder and create symlink to data folder
    os.chdir('original')
    if os.path.exists('data'): os.system('rm data')
    os.system('ln -s %s/data'%syspath)
    
    # Prepare input commands to generate chunks on VPFIT
    hdin = numpy.loadtxt('../header.dat',dtype=str,delimiter='\n')
    vpfit_commands(len(hdin))
    
    # Set up VPFIT files
    os.system('cp %s atom.dat'%atomdir)
    numpy.savetxt('vp_setup.dat',vpsetup,fmt="%s")
    os.environ['ATOMDIR']=os.path.abspath('./atom.dat')
    os.environ['VPFSETUP']=os.path.abspath('./vp_setup.dat')
    # Run VPFIT and create chunks for original fort.13
    os.system(vpfit+' < fitcommands > termout')
    
    # Remove temporary files and move chunks to specific folder
    os.system('rm fitcommands termout')
    os.system('mkdir -p chunks/')
    os.system('mv vpfit_chunk* chunks/')
    
    # Extract data chunks, modify error array and add noise
    shifts = numpy.empty((0,2))
    os.chdir('../')
    wa,fl,er = read_spec('original/data/'+spec)
    
    # Create artificial data
    for ireg in range(len(regs)):
        data  = numpy.loadtxt('original/chunks/vpfit_chunk%03i.txt'%(ireg+1),comments='!')
        # Initialize non-used parameters called in getshift function
        shift = float(getshift(regs[ireg,0],regs[ireg,1],slope,qso,sample,explist,settings))
        wmin  = data[0,0]
        wmax  = data[-1,0]
        imin1 = abs(wa-(wmin-3)).argmin()
        imin2 = abs(wa-wmin).argmin()
        imax1 = abs(wa-wmax).argmin()
        imax2 = abs(wa-(wmax+3)).argmin()
        wave  = wa[imin1:imax2+1]
        error = er[imin1:imax2+1]
        flux  = numpy.hstack(([1]*(imin2-imin1),data[:,3],[1]*(imax2-imax1)))
        ofile = open('data/spec%02i.txt'%(ireg+1),'w')
        for j in range(len(wave)):
            iwave  = wave[j]*(2*c+shift)/(2*c-shift)
            ierror = error[j] if snr==None else 1./snr
            iflux  = flux[j] if ierror<0 else flux[j]+numpy.random.normal(0,ierror)
            ofile.write('{0:>21} {1:>25} {2:>30}\n'.format('%.13f'%iwave,'%.16f'%iflux,'%.16E'%ierror))
        ofile.close()
        
    if expind:
        outfile = open('shifts.dat','w')
        for i in range(len(shifts)):
            outfile.write(shifts[i,0]+'  '+shifts[i,1]+'\n')
        outfile.close()




import sys,os,numpy
from matplotlib import re
from .simulation import create_simulated_model
from .utils import get_coddam,get_path_to_model,get_test_name,check_file_path
from .vpfit import vpfit_setup
from .forthandle import store_data,fort_update,prepare_model,create_fit13
from .constants import *
from .distortion import slopelist

def batch_run(target_system,selection,instrument,dbkey,simulation=False,model='turbulent',vpfit='vpfit',
              chisq=1.E-5,path='./',whitmore=False,stdev=0,mid_slope=0,sloperange=[None,None],step=0,
              snr=None,no_alpha=False,previous=False,arm=None,compress=False,repeat=1):
    """
    This operation will create realistic simulated absorption systems based
    on the real turbulent fort.13 model from each selected system.
    
    Examples
    --------
    
    >>> alpha run --system J000344-232354/0.45210/UVES_squader \ 
    >>>           --simulation complex --stdev 0.05 \ 
    >>>           --slope -0.123 --sloperange -1.123 0.877 \ 
    >>>           --step 0.1 --model 1 --previous
    
    The following is the command to be run to reproduce the results from
    `Dumont et al. (2017) <https://doi.org/10.1093/mnras/stx381>`_:

    >>> alpha run --system J043037m485523/1.35560/UVES_squader/ \ 
    >>>           --simulation complex --snr 1000 --slope 0.200 \ 
    >>>           --sloperange 0 0.3 --step 0.005 --model 7.2
    >>>           --chisq 1e-05 --previous --instrument uves
    """
    # Set up suitable lookup table
    if target_system!=None:
        database = {'system':[target_system],'whitmore':[whitmore]}
    elif simulation:
        database = get_coddam('distres',dbkey)
    elif selection!=None:
        database = get_coddam('curlist',dbkey)
    else:
        database = get_coddam('publist',dbkey)
    # Get VPFIT setup
    vpfit,atom,atomdir,vpsetup,vpfsetup = vpfit_setup(vpfit,instrument,chisq,no_alpha)
    os.chdir(path)
    path = os.getcwd()
    # Loop through database
    for i in range(len(database)):
        # Extract QSO name, absorption redshift and sample name from system name
        system = database['system'][i]
        qso    = system.split('/')[0]
        zabs   = system.split('/')[1]
        sample = system.split('/')[2]
        # Identify right target model
        simtest,modpath = get_path_to_model(system,target_system,simulation,whitmore,database,stdev,mid_slope,snr,model,selection)
        if modpath==None:
            continue
        # Define model directory name
        syspath = '/'.join([path,system,modpath])
        if os.path.exists(syspath)==False:
            print('ERROR: Path not found (%s)'%syspath)
            quit()
        print('|- System/Model: %s/%s'%(system,modpath))
        # Define run directory name
        runpath = '/'.join([syspath,'sims',simtest]) if simulation else syspath+'/runs'
        os.chdir(runpath)
        # If no simulation requested, create alias of model repository
        if simulation==False:
            if os.path.exists('model'):
                os.system('rm model')
            os.system('ln -s ../model model')
        # If simulation requested, create simulated data
        if simulation==True:
            print('|  |- Simulation: sims/%s'%simtest)
            if os.path.exists('/'.join([runpath,'model','original']))==False:
                # Move to target repository
                create_simulated_model(syspath,vpfit,atomdir,instrument,qso,sample,slope,snr,chisq,atomdir,vpsetup)
        # Prepare distortion repository and create synthetis spectrum if not existent
        test_name = get_test_name(instrument,step,mid_slope,chisq,previous=previous,whitmore=whitmore,simulation=simulation)
        # Fit absorption line system
        fit_system(test_name,vpfit,runpath,instrument,qso,sample,chisq,no_alpha,mid_slope,sloperange,
                   step,previous,arm,compress,simulation,whitmore,repeat,shape,stdev)
        
def fit_system(test_name='fit',vpfit='vpfit',runpath='./',instrument=None,qso=None,sample=None,chisq=1.E-5,no_alpha=False,mid_slope=0,sloperange=[None,None],step=0,previous=False,arm=None,compress=False,simulation=False,whitmore=False,repeat=1,shape='slope',explist=None,settings=None,stdev=0,mgisotope=False,atomdir=None,vpfsetup=None):

    print('|  |  |- Fitting version:',test_name.split('/')[-1])
    # Get absolute path for further calls
    explist,settings,atomdir,vpfsetup = check_file_path([explist,settings,atomdir,vpfsetup])
    # Move to run path directory
    runpath = os.path.abspath(runpath)
    # Get VPFIT setup
    vpfit,atom,atomdir,vpsetup,vpfsetup = vpfit_setup(vpfit,instrument,chisq,no_alpha,atomdir,vpfsetup)
    # Get full path of explist and settings files
    if explist!=None: os.path.abspath(explist)
    if settings!=None: os.path.abspath(settings)
    # Loop through all slope values and perform calculations
    distlist,distplot = slopelist(mid_slope,sloperange[0],sloperange[1],step)
    for slope2fit in distlist:
        # Create and move to distortion folder
        distortion = '0.000' if round(slope2fit,3)==0 else 'm%.3f'%abs(slope2fit) if slope2fit<0 else 'p%.3f'%abs(slope2fit)
        distpath = '%s/%s/%s'%(runpath,test_name,distortion)
        # Loop over models to prepare and fit them
        models = ['turbulent'] if simulation else ['turbulent','thermal']
        for model2fit in models:
            # Retrieve path to header file
            head_path = runpath+'/model/header.dat'
            # Retrieve path to fort.13 file
            if previous and mid_slope!=slope2fit and step!=0:
                i = slope2fit - step if slope2fit > mid_slope else slope2fit+step
                path_dist = '0.000' if round(i,3)==0 else 'm%.3f'%abs(i) if i<0 else 'p%.3f'%abs(i)
                fort_path = runpath+'/'+fit_name+'/'+path_dist+'/'+model2fit+'/'+model2fit+'_fit.13'
            else:
                fort_path = runpath+'/model/'+model2fit+'.13'
            # Modify fort.13 and header information
            fort_header_old,fort_content_old,comp,twoq = store_data(fort_path,head_path)
            fort_header_new,fort_content_new,qred,qblue = fort_update(fort_header_old,fort_content_old,atom,qso,sample,slope2fit,mgisotope,shape,whitmore,arm,explist,settings,stdev)
            # Check conditions and run fit if allowed
            skip1 = ( arm=='red' and len(qred)==0 )
            skip2 = ( arm=='blue' and len(qblue)==0 )
            skip3 = ( arm=='redblue' and len(qred)==0 and len(qblue)==0 )
            skip4 = ( sys.argv[1]=='run' and '--catchup' in sys.argv and os.path.exists(runpath+'/'+distortion+'/thermal'+modflag+'/fort.26')==True )
            skip5 = ( sys.argv[1]=='results' and system in outliers )
            if skip1 or skip2 or skip3 or skip4 or skip5:
                continue
            # Loop over number of VPFIT iterations
            for nrep in range(repeat):
                # Create fitting mode flag if several VPFIT iterations are requested
                modflag = '' if nrep==0 else '_%i'%(nrep+1)
                print('|  |  |- Distortion slope:',distpath.split('/')[-1])
                print('|  |  |  |- Processing %s fitting...'%model2fit)
                print('|  |  |  |- VPFIT iteration:',nrep+1)
                modpath = distpath+'/'+model2fit+modflag
                os.system('mkdir -p '+modpath)
                os.chdir(modpath)
                # Prepare input fort.13 for distortion slope
                prepare_model(model2fit,fort_header_new,fort_content_new,simulation)
                # Set up VPFIT files
                os.system('cp %s atom.dat'%atomdir)
                numpy.savetxt('vp_setup.dat',vpsetup,fmt="%s")
                os.environ['ATOMDIR']=os.path.abspath('./atom.dat')
                os.environ['VPFSETUP']=os.path.abspath('./vp_setup.dat')
                # Remove and re-create link to data folder
                if os.path.islink('data'): os.system('rm data')
                os.system('ln -s ../../../model/data')
                if '--iterone' in sys.argv:            
                    open('fitcommands','w').write('e\n\n\n'+model2fit+'_ini.13\nn\nn\n')
                elif '--illcond' in sys.argv:
                    open('fitcommands','w').write('f\nil\n\n\n'+model2fit+'_ini.13\nn\nn\n')
                else:
                    open('fitcommands','w').write('f\n\n\n'+model2fit+'_ini.13\nn\nn\n')
                os.system(vpfit+' < fitcommands > termout')
                if os.path.exists('fort.26')==True:
                    create_fit13(model2fit)
        os.chdir(runpath)
        # Compress results for supercomputer calculations
        if compress:
            destination = path.replace('list','execute')+'/'
            target   = system+'/'+model+'/sims/'+simtest+'/'+test+'/'+distortion
            filename = target.replace('/','--')
            os.system('tar -zcvf '+filename+'.tar.gz '+target+'/')
            os.system('rm -rf '+target+'/')
            os.system('mkdir -p '+destination)
            os.system('mv '+filename+'.tar.gz '+destination)



__version__ = '4.1.0'
from .fitting import batch_run,fit_system
from .simulation import create_simulated_model
from .curveplot import curveplot
from .metals import mgfe,qlist
from .utils import *
from .vpfit import *



import os,sys,numpy
from astropy.io import fits

def check_file_path(input_files):
    for i,input_file in enumerate(input_files):
        if input_file!=None:
            assert os.path.exists(input_file), 'File not found at %s not found. Abort.'%input_file
            input_files[i] = os.path.abspath(input_file)
    return input_files

def get_coddam(sheetname,dbkey):
    sheet_id = {'qsolist':26063364,'dipole':1491749328,'curlist':175866280,'publist':353421904,'distres':684542576}
    url = "https://docs.google.com/spreadsheets/d/{0}/export?format=csv&gid={1}".format(dbkey,sheet_id[sheetname])
    r = requests.get(url)
    sio = io.StringIO( r.text, newline=None)
    output = read_csv(sio,skiprows=1)
    return output

def get_fit_name(instrument,mode,sample):
    """
    Get fitting repository name. DEPRECATED!
    """
    name = instrument + '-'
    name = name + 'v10_slope'  if instrument=='hires'        else name
    name = name + 'v10_all'    if instrument=='uves'         else name
    name = name + '_jw'        if '--whitmore' in sys.argv   else name
    name = name + '_prev'      if '--previous' in sys.argv   else name
    name = name + '-' + mode   if mode in ['global','model'] else name
    name = name + '-' + sample if sample!=None               else name
    return name

def get_test_name(instrument=None,distsep=None,slope=0,chisq=1.E-5,vpversion='10',arm=None,shape='slope',
                 previous=False,mgisotope=False,murphy=False,whitmore=False,simulation=False,newatom=False):
    '''
    Create distortion calculation test repository name.

    Parameters
    ----------
    instrument : str
      Spectrograph to be used
    distsep : float
      Step size value
    slope : float
      Middle or single slope value
    chisq : float
      Chi-square fitting value
    vpversion : str
      VPFIT version
    arm : str
      Spectrograph arm to select
    shape : str
      Shape of distortion model
    previous : bool
      Used previous slope results as first guesses
    mgisotope : bool
      Do non-terrestrial MgII isotope calculations
    murphy : bool
      Use original vp_setup.dat from Murphy from early 2000's
    whitmore : bool
      Use simplistic distortion model from Whitmore
    simulation : bool
      Do simulated distortion calculations
    newatom : bool
      Use new atomic data
    
    Return
    ------
    test : str
      Distortion calculation folder name
    '''
    test = 'v' + vpversion + '_chisq%.E'%chisq
    test = test+'_'+arm   if instrument=='uves'  and simulation==False        else test
    test = test+'_'+shape if instrument=='hires' and (slope!=0 or distsep!=0) else test
    test = test+'_step%.E'%distsep if distsep!=0                              else test
    test = test+'_prev'   if previous and (slope!=0 or distsep!=0)            else test
    test = test+'_jw'     if whitmore and (slope!=0 or distsep!=0)            else test
    test = test+'_null'   if simulation==False and noalpha                    else test
    test = test+'_new'    if newatom                                          else test
    test = test+'_murphy' if murphy                                           else test
    test = test+'_mg'     if mgisotope                                        else test
    return test

def get_path_to_model(system,target_system,simulation,whitmore,database,stdev,mid_slope,snr,target_model,selection):
    simtest,model = None,None
    if simulation and system==target_system and target_system not in outliers and (database['whitmore'][i]=='yes')==whitmore:
        sim = 'sim2' if stdev!=0 else 'sim1'
        slope = float(database['%s mid%02i'%(sim,100*stdev)][i]) if mid_slope==None else mid_slope
        tilt = '0.000' if round(slope,3)==0 else 'm%.3f'%abs(slope) if slope<0 else 'p%.3f'%abs(slope)
        simtest = 'complex_%s_%.2f'%(tilt,stdev)
        simtest = simtest if snr==None else simtest+'_snr%.E'%snr
        model = database['%s model'%sim][i] if target_model==None else target_model
        model = 'model-%.1f'%float(model)
    if target_system!=None and target_system==system:
        model = target_model if selection==None else database[selection][i]
        model = None if model==None else 'model-%.1f'%float(model)
    if selection!=None and database[selection][i]!='-':
        model = database[selection][i]
        model = 'model-%.1f'%float(model)
    if instrument==database['instrument'][i].lower():
        model = 'model-7.1'
    return simtest,model

def get_system_list():
    """
    Create list of systems based on user-defined selection criteria.
    """
    n = 0
    filename = 'fortlist_'+str(system).replace('/','--')+'.dat'
    os.system('find '+fitdir+' -mindepth 3 -maxdepth 3 | sort > list_'+filename)
    allsys = numpy.loadtxt('list_'+filename,dtype='str')
    outfile = open(filename,'w')
    for line in allsys:
        quasar  = line.split('/')[-3]
        redabs  = line.split('/')[-2]
        sample  = line.split('/')[-1]
        system  = quasar+'/'+redabs+'/'+sample
        # condition 0: check if system specified by the user and match the current system
        cond0   = system==None or system==system
        # condition 1: either sample is specified or in the list of Keck+VLT sample
        cond1   = sample==sample or (sample==None and sample in keck+vlt)
        # condition 2: either selection not specified or valid selection name and identified model
        cond2a  = selection==None
        cond2b  = str(selection) in curlist.head(0).columns.values and \
                  system in numpy.array(curlist['system']) and \
                  curlist[selection][numpy.where(numpy.array(curlist['system'])==system)[0][0]]!='-'
        # condition 3: check user-defined instrument and identify right sample
        cond3a  = instrument==None
        cond3b  = instrument=='uves'  and sample in vlt
        cond3c  = instrument=='hires' and sample in keck
        # condition 4: if clip option specified, check that the system is in the clipped list
        cond4   = ('--clip' not in sys.argv) or ('--clip' in sys.argv and system not in clippeduves)
        if cond0 and cond1 and (cond2a or cond2b) and (cond3a or cond3b or cond3c) and cond4:
            # Get the model ID
            if '--distortion' in sys.argv and selection=='jak12':
                whitmore = 'yes' if '--whitmore' in sys.argv else 'no'
                i = numpy.where(numpy.logical_and(distres['system']==system,distres['whitmore']==whitmore))[0][0]
                model = 'model-%.1f'%float(distres['model individual'][i])
            elif selection!=None:
                model = 'model-%.1f'%float(curlist[selection][numpy.where(curlist['system']==system)[0][0]])
            elif model!=None:
                model = 'model-%.1f'%float(model)
            else:
                print('ERROR: Please specify either --distortion, the model or the selection...')
                quit()
            # Add system into list
            if simulation:
                outfile.write(line+'/'+model+'/sims/'+simtest+'/'+test+'/'+distortion+'\n')
            else:
                outfile.write(line+'/'+model+'/runs/'+test+'/'+distortion+'\n')
            n = n + 1
    outfile.close()
    os.system('rm list_'+filename)
    if n==0:
        print('|- ABORT: No absorption systems found, please make sure your arguments are correct.')
        quit()
    return filename

def read_spec(specfile):
    '''
    Read spectrum and extract wavelength and flux dataset.
    '''
    datatype = specfile.split('.')[-1]
    wavefile = specfile.replace('.fits','.wav.fits')
    if datatype=='fits' and os.path.exists(wavefile)==True:
        fh = fits.open(wavefile)
        hd = fh[0].header
        specwa = fh[0].data
        fh = fits.open(specfile)
        hd = fh[0].header
        specfl = fh[0].data        
    elif datatype=='fits':
        fh = fits.open(specfile)
        hd = fh[0].header
        d  = fh[0].data
        if ('CTYPE1' in hd and hd['CTYPE1'] in ['LAMBDA','LINEAR']) or ('DC-FLAG' in hd and hd['DC-FLAG']=='0'):
            specwa = hd['CRVAL1'] + (hd['CRPIX1'] - 1 + numpy.arange(hd['NAXIS1']))*hd['CDELT1']
        else:
            specwa = 10**(hd['CRVAL1'] + (hd['CRPIX1'] - 1 + numpy.arange(hd['NAXIS1']))*hd['CDELT1'])
        if len(d.shape)==1:
            specfl = d[:]
            specer = None
        else:
            specfl = d[0,:]
            specer = d[1,:]
    else:
        sp = numpy.loadtxt(specfile,comments='!')
        specwa = sp[:,0]
        specfl = sp[:,0]
        specer = sp[:,2]
    return specwa,specfl,specer



from distutils.core import setup
import os,alphaqso.__init__
from glob import glob

def get_data_names(root):
    '''
    Return list of all filenames (not directories) under root.
    '''
    temp = [root+'/*']
    for dirpath, dirnames, filenames in os.walk(root):
        temp.extend((os.path.join(dirpath, d, '*') for d in dirnames))
    names = []
    for path in temp:
        if any(os.path.isfile(f) for f in glob(path)):
            names.append(path.replace('alphaqso/',''))
    return names

package_data = {'alphaqso' : get_data_names('alphaqso/data')}

setup(
    name="alphaqso",
    version=alphaqso.__version__,
    author="Vincent Dumont",
    author_email="vincentdumont11@gmail.com",
    packages=["alphaqso"],
    requires=["numpy","matplotlib","scipy","astropy"],
    package_data = package_data,
    include_package_data=True,
    scripts = glob('bin/*'),
    url="https://astroquasar.gitlab.io/programs/alphaqso",
    description="Fine-structure constant measurement and distortion analysis tools.",
    install_requires=[]
)



import matplotlib,os,sys,re,math,datetime,operator,random,ctypes,binascii
import numpy                         as np
import matplotlib                    as mpl
import matplotlib.dates              as mdates
import matplotlib.pyplot             as plt
from matplotlib.pyplot               import *
from pylab                           import *
from matplotlib                      import rc
from matplotlib._png                 import read_png
from matplotlib.offsetbox            import AnnotationBbox, OffsetImage
from mpl_toolkits.axes_grid1         import make_axes_locatable
from scipy                           import optimize,stats
from scipy.optimize                  import curve_fit
from matplotlib.backends.backend_pdf import PdfPages
from time                            import clock
from scipy.ndimage                   import gaussian_filter1d
import pandas                        as pd
import theano.tensor                 as t
from astropy.io                      import fits
from mpl_toolkits.basemap            import Basemap

def getmetals():
    data = np.empty((0,7),dtype=float)
    metals = np.array([['MgII' ,np.empty((0,1),dtype=str),np.empty((0,5),dtype=float)],
                       ['FeII' ,np.empty((0,1),dtype=str),np.empty((0,5),dtype=float)],
                       ['SiII' ,np.empty((0,1),dtype=str),np.empty((0,5),dtype=float)],
                       ['NiII' ,np.empty((0,1),dtype=str),np.empty((0,5),dtype=float)],
                       ['CrII' ,np.empty((0,1),dtype=str),np.empty((0,5),dtype=float)],
                       ['AlII' ,np.empty((0,1),dtype=str),np.empty((0,5),dtype=float)],
                       ['AlIII',np.empty((0,1),dtype=str),np.empty((0,5),dtype=float)],
                       ['MnII' ,np.empty((0,1),dtype=str),np.empty((0,5),dtype=float)]],dtype=object)
    for system in np.loadtxt(self.pathdata+'metallist.dat',dtype=str):
        self.quasar   = system.split('/')[0]
        self.zabs     = float(system.split('/')[1])
        self.sample   = system.split('/')[2]
        self.distpath = './pubsys/'+system+'/'
        if os.path.exists(self.distpath+'chunks/vpfit_chunk001.txt')==True:
            dv   = []
            k    = 0
            tied = 0
            flag = 0
            
            header = np.loadtxt(self.distpath+'header.dat',dtype=str,delimiter='\n')
            fort13 = np.loadtxt(self.distpath+'turbulent.13',dtype=str,delimiter='\n')
            for line in fort13:
                vals = line.split()
                flag = 1 if '*' in line and flag==0 else 2 if '*' in line and flag==1 else flag
                if len(vals)==0:
                    break
                if flag==1 and '*' not in line:
                    if 'external' not in header[k]:
                        trans = header[k].split()[0]
                        wmid  = self.atominfo(trans)[1]
                        dv.append(2*(float(vals[3])-float(vals[2]))/(float(vals[2])+float(vals[3]))*self.c)
                    k += 1
                if flag==2 and '*' not in line:
                    alpha = vals[4] if len(vals[0])==1 else vals[3]
                    zabs  = vals[3] if len(vals[0])==1 else vals[2]
                    zabs  = float(zabs[:-2]+re.compile(r'[^\d.-]+').sub('',zabs[-2:]))
                    tied  = tied + 1 if alpha[-1].islower()==True and abs(zabs-self.zabs)<0.003 else tied
                    ion   = vals[0]+vals[1] if len(vals[0])==1 else vals[0]
                    if ion in metals[:,0] and alpha[-1].isdigit()==False and abs(zabs-self.zabs)<0.003:
                        i = np.where(metals[:,0]==ion)[0][0]
                        if system not in metals[i,1]:
                            metals[i,1] = np.vstack((metals[i,1],[str(system)]))
                            metals[i,2] = np.vstack((metals[i,2],[1,0,self.zabs,0,0]))
                        else:
                            j = np.where(metals[i,1]==system)[0][0]
                            metals[i,2][j,0] += 1
                            
            os.system('ls '+self.distpath+'chunks/ > list')
            chunks = np.loadtxt('list',dtype=str)
            noise,ewidth = [],[]
            for n in range(len(chunks)):
                ion   = header[n].split()[0].split('_')[0]
                chunk = np.loadtxt(self.distpath+'chunks/'+chunks[n],comments='!')
                chunk = np.delete(chunk,np.where(chunk[:,2]==0)[0],0)
                width = sum([(1-chunk[i,-1])*(chunk[i+1,0]-chunk[i,0])/(self.zabs+1) for i in range(len(chunk)-1)])
                snr   = np.average(chunk[:,1]/chunk[:,2])
                if 'external' not in header[n] and ion in metals[:,0]:
                    i = np.where(metals[:,0]==ion)[0][0]
                    if system in metals[i,1]:
                        j = np.where(metals[i,1]==system)[0][0]
                        metals[i,2][j,1] += width
                        metals[i,2][j,3] += snr
                        metals[i,2][j,4] += 1.
                ewidth.append(width)
                noise.append(snr)
                
            for i in range(len(metals)):
                if system in metals[i,1]:
                    j = np.where(metals[i,1]==system)[0][0]
                    if metals[i,2][j,1]>0:
                        metals[i,2][j,1] = metals[i,2][j,1] / metals[i,2][j,4]
                        metals[i,2][j,4] = 1
                
            data = np.vstack((data,np.array([float(tied),
                                             np.mean(ewidth),
                                             np.std(ewidth),
                                             self.zabs,
                                             np.mean(noise),
                                             np.mean(dv),
                                             np.std(dv)],dtype=float)))
            
    os.system('rm list')
    
    ''' Bin data '''
    
    k = 1
    data = np.array(sorted(data,key=lambda col: col[3]))
    bindata = np.empty((0,7))
    for i in range(0,len(data),self.binning):
        ilim  = i+self.binning if i+self.binning<=len(data) else len(data)
        print '\nbin',k,'\n'
        for j in range(i,ilim):
            print 'z={0:<8}  |  {1:>5}  |  {2:>5} +/- {3:<6}  |  {4:<8}'.format('%.5f'%data[j,3],'%i'%data[j,0],'%.2f'%data[j,1],'%.2f'%data[j,2],'%.2f'%data[j,4])
        zabs  = np.average([float(data[j,3]) for j in range(i,ilim)])
        tied  = np.average([float(data[j,0]) for j in range(i,ilim)])
        snr   = np.average([float(data[j,4]) for j in range(i,ilim)])
        width = sum([float(data[j,1])/float(data[j,2])**2 for j in range(i,ilim)]) / sum([1/float(data[j,2])**2 for j in range(i,ilim)])
        error = 1 / np.sqrt(sum(1/float(data[j,2])**2 for j in range(i,ilim)))
        dv    = sum([float(data[j,5])/float(data[j,6])**2 for j in range(i,ilim)]) / sum([1/float(data[j,6])**2 for j in range(i,ilim)])
        dverr = 1 / np.sqrt(sum(1/float(data[j,6])**2 for j in range(i,ilim)))
        print '----------  |  -----  |  ----------------  |  --------'
        print 'z={0:<8}  |  {1:>5}  |  {2:>5} +/- {3:<6}  |  {4:<8}'.format('%.5f'%zabs,'%i'%tied,'%.2f'%width,'%.2f'%error,'%.2f'%snr)
        bindata   = np.vstack((bindata,[tied,width,error,zabs,snr,dv,dverr]))
        k += 1

    return bindata,metals

def isfloat(value):
    try:
      float(value)
      return True
    except ValueError:
      return False

#==================================================================================================================

def makeatomlist(atompath):     # Store data from atom.dat

    atom   = np.empty((0,6))
    atomdat     = np.loadtxt(atompath,dtype='str',delimiter='\n')
    for element in atomdat:
        l       = element.split()
        i       = 0      if len(l[0])>1 else 1
        species = l[0]   if len(l[0])>1 else l[0]+l[1]
        wave    = 0 if len(l)<i+2 else 0 if isfloat(l[i+1])==False else l[i+1]
        f       = 0 if len(l)<i+3 else 0 if isfloat(l[i+2])==False else l[i+2]
        gamma   = 0 if len(l)<i+4 else 0 if isfloat(l[i+3])==False else l[i+3]
        mass    = 0 if len(l)<i+5 else 0 if isfloat(l[i+4])==False else l[i+4]
        alpha   = 0 if len(l)<i+6 else 0 if isfloat(l[i+5])==False else l[i+5]
        if species not in ['>>','<<','<>','__']:
            atom = np.vstack((atom,[species,wave,f,gamma,mass,alpha]))
    return atom
            
#==================================================================================================================

def atominfo(atomID):     # Get atomic data from selected atomID

    target = [0,0,0,0,0]
    atomID = atomID.split('_')
    for i in range(len(self.atom)):
        element     = self.atom[i,0]
        wavelength  = self.atom[i,1]
        oscillator  = self.atom[i,2]
        gammavalue  = self.atom[i,3]
        qcoeff      = self.atom[i,5]
        if (len(atomID)>1 and element==atomID[0] and abs(float(wavelength)-float(atomID[1]))<abs(float(target[1])-float(atomID[1]))) \
           or (len(atomID)==1 and element==atomID[0]):
           target = [element,wavelength,oscillator,gammavalue,qcoeff] 
    if target==[0,0,0,0,0]:
        print atomID,'not identifiable...'
        quit()
    return target

#==================================================================================================================

def plotwidth():

    def func(x,a,b):
        return a + b*x
        
    bindata,metals = getmetals()


    ''' ----------------------------------------------------- '''
    ''' Plot Average Equivalent Width vs. Velocity Dispersion '''
    ''' ----------------------------------------------------- '''


    x    = bindata[:,5]
    xerr = bindata[:,6]
    y    = bindata[:,1]
    yerr = bindata[:,2]
    zabs = bindata[:,3]

    xmax = 1.1*max(x)
    ymax = 0.2#1.1*max(y)
    
    fig = figure(figsize=(7,6))
    plt.subplots_adjust(left=0.1, right=0.87, bottom=0.1, top=0.95, hspace=0, wspace=0)

    ax = subplot(111,xlim=[0,xmax],ylim=[0,ymax])
    ax.scatter(x,y,marker='o',s=50,edgecolors='none',zorder=3,\
            c=zabs,cmap=mpl.cm.cool,vmin=min(zabs),vmax=max(zabs))
    errorbar(x,y,yerr=yerr,fmt='o',ms=0,c='0.7',zorder=1)
    xlabel('Average velocity dispersion',fontsize=12)
    ylabel('Average equivalent width',fontsize=12)
    axhline(y=0,ls='dotted',color='black')
    axvline(x=0,ls='dotted',color='black')
    
    xfit = np.arange(0,xmax,0.01)
    coeffs,matcov = curve_fit(func,x,y)
    yfit = func(xfit,coeffs[0],coeffs[1])
    ax.plot(xfit,yfit,color='black',lw=3,ls='dotted',
            label=r'Unweighted fit: $%.6f \pm %.6f$ '%(coeffs[1],np.sqrt(matcov[1][1])))
    
    xfit = np.arange(0,xmax,0.01)
    coeffs,matcov = curve_fit(func,x,y,sigma=yerr)
    yfit = func(xfit,coeffs[0],coeffs[1])
    ax.plot(xfit,yfit,color='black',lw=3,ls='dashed',
            label=r'Weighted fit: $%.6f \pm %.6f$ '%(coeffs[1],np.sqrt(matcov[1][1])))
    
    leg = plt.legend(fancybox=True,loc=2,numpoints=1,handlelength=3,prop={'size':12})
    leg.get_frame().set_linewidth(0.1)
                
    ax1  = fig.add_axes([0.87,0.1,0.04,0.85])
    cmap = mpl.cm.cool
    norm = mpl.colors.Normalize(vmin=min(zabs),vmax=max(zabs))
    cb1  = mpl.colorbar.ColorbarBase(ax1,cmap=cmap,norm=norm)
    cb1.set_label('Absorption redshift',fontsize=12)
    
    savefig('equiwidth_vs_dispersion.pdf')
    clf()

    
    ''' ----------------------------------------------------------- '''
    ''' Plot Average Equivalent Width vs. Number of Tied Components '''
    ''' ----------------------------------------------------------- '''


    x    = bindata[:,0]
    y    = bindata[:,1]
    yerr = bindata[:,2]
    zabs = bindata[:,3]
    
    xmax = 1.1*max(x)
    ymax = 1.1*max(y)
    
    fig = figure(figsize=(7,6))
    plt.subplots_adjust(left=0.1, right=0.87, bottom=0.1, top=0.95, hspace=0, wspace=0)

    ax = subplot(111,xlim=[0,xmax],ylim=[0,ymax])
    ax.scatter(x,y,marker='o',s=50,edgecolors='none',zorder=3,\
            c=zabs,cmap=mpl.cm.cool,vmin=min(zabs),vmax=max(zabs))
    errorbar(x,y,yerr=yerr,fmt='o',ms=0,c='0.7',zorder=1)
    xlabel('Number of tied components',fontsize=12)
    ylabel('Average equivalent width',fontsize=12)
    axhline(y=0,ls='dotted',color='black')
    axvline(x=0,ls='dotted',color='black')
    
    xfit = np.arange(0,xmax,0.01)
    coeffs,matcov = curve_fit(func,x,y)
    yfit = func(xfit,coeffs[0],coeffs[1])
    ax.plot(xfit,yfit,color='black',lw=3,ls='dotted',
            label=r'Unweighted fit: $%.6f \pm %.6f$ '%(coeffs[1],np.sqrt(matcov[1][1])))
    
    xfit = np.arange(0,xmax,0.01)
    coeffs,matcov = curve_fit(func,x,y,sigma=yerr)
    yfit = func(xfit,coeffs[0],coeffs[1])
    ax.plot(xfit,yfit,color='black',lw=3,ls='dashed',
            label=r'Weighted fit: $%.6f \pm %.6f$ '%(coeffs[1],np.sqrt(matcov[1][1])))
    
    leg = plt.legend(fancybox=True,loc=2,numpoints=1,handlelength=3,prop={'size':12})
    leg.get_frame().set_linewidth(0.1)
                
    ax1  = fig.add_axes([0.87,0.1,0.04,0.85])
    cmap = mpl.cm.cool
    norm = mpl.colors.Normalize(vmin=min(zabs),vmax=max(zabs))
    cb1  = mpl.colorbar.ColorbarBase(ax1,cmap=cmap,norm=norm)
    cb1.set_label('Absorption redshift',fontsize=12)
    
    savefig('equiwidth_vs_tied.pdf')
    clf()

    
    ''' ------------------------------------------- '''
    ''' Plot Number of tied components vs. Redshift '''
    ''' ------------------------------------------- '''


    x    = bindata[:,3]
    y    = bindata[:,0]
    c    = bindata[:,4]
        
    xmax = 1.1*max(x)
    ymax = 1.1*max(y)
    
    fig = figure(figsize=(7,6))
    plt.subplots_adjust(left=0.1, right=0.87, bottom=0.1, top=0.95, hspace=0, wspace=0)
    
    ax = subplot(111,xlim=[0,xmax],ylim=[0,ymax])
    ax.scatter(x,y,marker='o',s=50,edgecolors='none',zorder=3,\
               c=c,cmap=mpl.cm.rainbow,vmin=min(c),vmax=max(c))
    xlabel('Absorption redshift',fontsize=12)
    ylabel('Number of tied components',fontsize=12)
    axhline(y=0,ls='dotted',color='black')
    axvline(x=0,ls='dotted',color='black')

    xfit = np.arange(0,xmax,0.01)
    coeffs,matcov = curve_fit(func,x,y)
    yfit = func(xfit,coeffs[0],coeffs[1])
    ax.plot(xfit,yfit,color='black',lw=3,ls='dashed',
            label=r'Unweighted fit: $%.6f \pm %.6f$ '%(coeffs[1],np.sqrt(matcov[1][1])))
    
    leg = plt.legend(fancybox=True,loc=2,numpoints=1,handlelength=3,prop={'size':12})
    leg.get_frame().set_linewidth(0.1)
                
    ax1  = fig.add_axes([0.87,0.1,0.04,0.85])
    cmap = mpl.cm.rainbow
    norm = mpl.colors.Normalize(vmin=min(c),vmax=max(c))
    cb1  = mpl.colorbar.ColorbarBase(ax1,cmap=cmap,norm=norm)
    cb1.set_label('Signal-to-Noise ratio',fontsize=12)
    
    savefig('tiedcomp_vs_redshift.pdf')
    clf()

    
    ''' ------------------------------------------ '''
    ''' Plot Average Equivalent Width vs. Redshift '''
    ''' ------------------------------------------ '''

    
    x    = bindata[:,3]
    y    = bindata[:,1]
    yerr = bindata[:,2]
    c    = bindata[:,4]
        
    xmax = 1.1*max(x)
    ymax = 1.1*max(y)
    
    fig = figure(figsize=(7,6))
    plt.subplots_adjust(left=0.1, right=0.87, bottom=0.1, top=0.95, hspace=0, wspace=0)
    
    ax = subplot(111,xlim=[0,xmax],ylim=[0,ymax])
    ax.scatter(x,y,marker='o',s=50,edgecolors='none',zorder=3,\
               c=c,cmap=mpl.cm.rainbow,vmin=min(c),vmax=max(c))
    errorbar(x,y,yerr=yerr,fmt='o',ms=0,c='0.7',zorder=1)
    xlabel('Absorption redshift',fontsize=12)
    ylabel('Average Equivalent Width',fontsize=12)
    axhline(y=0,ls='dotted',color='black')
    axvline(x=0,ls='dotted',color='black')

    xfit = np.arange(0,xmax,0.01)
    coeffs,matcov = curve_fit(func,x,y)
    yfit = func(xfit,coeffs[0],coeffs[1])
    ax.plot(xfit,yfit,color='black',lw=3,ls='dotted',
            label=r'Unweighted fit: $%.6f \pm %.6f$ '%(coeffs[1],np.sqrt(matcov[1][1])))
    
    xfit = np.arange(0,xmax,0.01)
    coeffs,matcov = curve_fit(func,x,y,sigma=yerr)
    yfit = func(xfit,coeffs[0],coeffs[1])
    ax.plot(xfit,yfit,color='black',lw=3,ls='dashed',
            label=r'Weighted fit: $%.6f \pm %.6f$ '%(coeffs[1],np.sqrt(matcov[1][1])))
    
    leg = plt.legend(fancybox=True,loc=2,numpoints=1,handlelength=3,prop={'size':12})
    leg.get_frame().set_linewidth(0.1)
                
    ax1  = fig.add_axes([0.87,0.1,0.04,0.85])
    cmap = mpl.cm.rainbow
    norm = mpl.colors.Normalize(vmin=min(c),vmax=max(c))
    cb1  = mpl.colorbar.ColorbarBase(ax1,cmap=cmap,norm=norm)
    cb1.set_label('Signal-to-Noise ratio',fontsize=12)
    
    savefig('equiwidth_vs_redshift.pdf')
    clf()


#==================================================================================================================

def plotwidthperion():

    def func(x,a,b):
        return a + b*x
        
    bindata,metals = getmetals()

    ''' ------------------------------------------- '''
    ''' Plot number of tied components vs. redshift '''
    ''' ------------------------------------------- '''

    
    fig = figure(figsize=(8,12))
    plt.subplots_adjust(left=0.07, right=0.93, bottom=0.05, top=0.93, hspace=0.3, wspace=0.1)
    fig.suptitle('Number of tied components vs. Absorption redshift',fontsize=15)

    for i in range(len(metals)):

        ion   = metals[i,0]
        data  = np.array(sorted(metals[i,2],key=lambda col: col[2]))
        bdata = np.empty((0,2))
        for j in range(0,len(data),self.binning):
            jlim  = j+self.binning if j+self.binning<=len(data) else len(data)
            zabs  = np.average([float(data[k,2]) for k in range(j,jlim)])
            tied  = np.average([float(data[k,0]) for k in range(j,jlim)])
            bdata = np.vstack((bdata,[zabs,tied]))
            
        ax = subplot(4,2,i+1,xlim=[0,max(bdata[:,0])],ylim=[0,max(bdata[:,1])])
        ax.scatter(bdata[:,0],bdata[:,1],marker='o',s=40,edgecolors='none',zorder=3,c='black',alpha=0.6)
        axhline(y=0,ls='dotted',color='black')
        axvline(x=0,ls='dotted',color='black')

        if len(bdata)>1:
            xfit = np.arange(0,max(bdata[:,0]),0.01)
            coeffs,matcov = curve_fit(func,bdata[:,0],bdata[:,1])
            yfit = func(xfit,coeffs[0],coeffs[1])
            ax.plot(xfit,yfit,color='red',lw=2,ls='dashed')
            self.linslopeval = coeffs[1]
            self.linslopeerr = np.sqrt(matcov[1][1])
            #print metals[i,0]+r': %.4f +/- %.4f'%(self.linslopeval,self.linslopeerr)

        plt.title(ion,fontsize=12,color='red')

        if (i+1)%2==0:
            ax.yaxis.tick_right()
            ax.yaxis.set_ticks_position('both')
                    
    savefig('tied_vs_redshift_per_ion.pdf')
    clf()

    
    ''' ----------------------------------------------------------- '''
    ''' Plot average equivalent width vs. number of tied components '''
    ''' ----------------------------------------------------------- '''

    
    fig = figure(figsize=(8,12))
    plt.subplots_adjust(left=0.07, right=0.93, bottom=0.05, top=0.93, hspace=0.3, wspace=0.1)
    fig.suptitle('Average Equivalent Width vs. Number of tied components',fontsize=15)

    for i in range(len(metals)):

        ion   = metals[i,0]
        data  = np.array(sorted(metals[i,2],key=lambda col: col[0]))
        bdata = np.empty((0,2))
        for j in range(0,len(data),self.binning):
            jlim  = j+self.binning if j+self.binning<=len(data) else len(data)
            tied  = np.average([float(data[k,0]) for k in range(j,jlim)])
            width = np.average([float(data[k,1]) for k in range(j,jlim)])
            bdata = np.vstack((bdata,[tied,width]))
            
        ax = subplot(4,2,i+1,xlim=[0,max(bdata[:,0])],ylim=[0,max(bdata[:,1])])
        ax.scatter(bdata[:,0],bdata[:,1],marker='o',s=40,edgecolors='none',zorder=3,c='black',alpha=0.6)
        axhline(y=0,ls='dotted',color='black')
        axvline(x=0,ls='dotted',color='black')

        if len(bdata)>1:
            xfit = np.arange(0,max(bdata[:,0]),0.01)
            coeffs,matcov = curve_fit(func,bdata[:,0],bdata[:,1])
            yfit = func(xfit,coeffs[0],coeffs[1])
            ax.plot(xfit,yfit,color='red',lw=2,ls='dashed')

        plt.title(ion,fontsize=12,color='red')

        if (i+1)%2==0:
            ax.yaxis.tick_right()
            ax.yaxis.set_ticks_position('both')
                    
    savefig('equiwidth_vs_tied_per_ion.pdf')
    clf()

    
    ''' ------------------------------------------ '''
    ''' Plot average equivalent width vs. redshift '''
    ''' ------------------------------------------ '''

    
    fig = figure(figsize=(8,12))
    plt.subplots_adjust(left=0.07, right=0.93, bottom=0.05, top=0.93, hspace=0.3, wspace=0.1)
    fig.suptitle('Average Equivalent Width vs. Absorption redshift',fontsize=15)

    for i in range(len(metals)):

        ion   = metals[i,0]
        data  = np.array(sorted(metals[i,2],key=lambda col: col[2]))
        bdata = np.empty((0,2))
        for j in range(0,len(data),self.binning):
            jlim  = j+self.binning if j+self.binning<=len(data) else len(data)
            zabs  = np.average([float(data[k,2]) for k in range(j,jlim)])
            width = np.average([float(data[k,1]) for k in range(j,jlim)])
            bdata = np.vstack((bdata,[zabs,width]))
            
        ax = subplot(4,2,i+1,xlim=[0,max(bdata[:,0])],ylim=[0,max(bdata[:,1])])
        ax.scatter(bdata[:,0],bdata[:,1],marker='o',s=40,edgecolors='none',zorder=3,c='black',alpha=0.6)
        axhline(y=0,ls='dotted',color='black')
        axvline(x=0,ls='dotted',color='black')
        
        if len(bdata)>1:
            xfit = np.arange(0,max(bdata[:,0]),0.01)
            coeffs,matcov = curve_fit(func,bdata[:,0],bdata[:,1])
            yfit = func(xfit,coeffs[0],coeffs[1])
            ax.plot(xfit,yfit,color='red',lw=2,ls='dashed')

        plt.title(ion,fontsize=12,color='red')

        if (i+1)%2==0:
            ax.yaxis.tick_right()
            ax.yaxis.set_ticks_position('both')
                    
    savefig('equiwidth_vs_redshift_per_ion.pdf')
    clf()

if __name__ == "__main__":

    rc('font', size=2, family='sans-serif')
    rc('axes', labelsize=8, linewidth=0.2)
    rc('legend', fontsize=2, handlelength=10)
    rc('xtick', labelsize=10)
    rc('ytick', labelsize=10)
    rc('lines', lw=0.2, mew=0.2)
    rc('grid', linewidth=0.2)
    home     = os.getenv('HOME')+'/ASTRO/analysis/alpha'
    here     = os.getenv('PWD')
    pathdata = os.path.abspath(__file__).rsplit('/',1)[0] + '/data/'
    done     = False
    c        = 299792.458
    atom     = makeatomlist(pathdata+'atom.dat')
    argument = np.array(sys.argv, dtype='str')
    binning  = 12 if '--bin' not in sys.argv else int(argument[np.where(argument=='--bin')[0][0]+1])
    plotwidth()
    plotwidthperion()
    



import os,sys,numpy,re
from .vpfit import make_atom_list,atomic_info
from .distortion import getshift

def create_fit13(model,simulation=False):
    """
    Create model_fit.13 after fitting model_ini.13 completed.

    Parameters
    ----------
    model : string
      Fitting model, either thermal or turbulent
    simulation : bool (False)
      Set if the model is based on a simulation
    """
    flag26 = flag18 = 0
    final = open(model+'_fit.13','w')
    final.write('   *\n')
    line26 = numpy.loadtxt('fort.26',dtype='str',delimiter='\n')
    for i in range(len(line26)):
        if 'Stats:' in line26[i]:
            flag26 = 1
        if line26[i][0:2]!='%%':
            break
        else:
            final.write(line26[i].replace('%% ','')+'\n')
    final.write('  *\n')
    line18 = numpy.loadtxt('fort.18',dtype='str',delimiter='\n')
    for i in range(len(line18)-1,0,-1):
        if 'statistics for whole fit:' in line18[i]:
            flag18 = 1
        if 'chi-squared' in line18[i]:
            chisq   = '%.4f'%float(line18[i].split('(')[1].split(',')[0])
            chisqnu = '%.3f'%float(line18[i].split('(')[0].split(':')[1])
            ndf     = '%.0f'%float(line18[i].split(')')[0].split(',')[1])
            if simulation==False:
                print('|  |  |  |  |- chisq='+chisq+', ndf='+ndf+', chisq_nu='+chisqnu)
            a = i + 2
            break
    for i in range(a,len(line18)):
        if len(line18[i])==1:
            break
        final.write(line18[i]+'\n')
    final.close()
    if '--raijin' in sys.argv and simulation==False:
        os.system('cp '+model+'_fit.13 ../')
    if flag26==1 and flag18==1 and simulation==False:
        os.system('mv fort.26 '+model+'.26')
        os.system('mv fort.18 '+model+'.18')

def read_fort26(fortpath,daoaun):
    fort26 = numpy.loadtxt(fortpath,dtype='str',comments='%',delimiter='\n')[1:]
    for i,line in enumerate(fort26):
        s = 1 if len(line.split()[0])==1 else 0
        zabs = float(re.compile(r'[^\d.-]+').sub('',line.split()[1+s]))
        if 'q' in str(line.split()[7+s]) and abs(2*(zabs-float(zabs))/(zabs+float(zabs))) < 0.02:
            alpha = float(line.split()[7+s].split('q')[0])*daoaun/1e-5
            error = float(line.split()[8+s].split('q')[0])*daoaun/1e-5
            break
    return i,alpha,error
        
def read_fort18(fortpath,published=False):
    """
    Extract fitting results from fort.18 output. If the path to the fort.18 file is found,
    the routine will read the file from the end backward.
    """
    flag,n,df,chisq,chisq_nu,alpha,error,n,df = 1,'-','-','-','-','-','-','-','-'
    if fortpath!=None and os.path.exists(fortpath):
        daoaun = 1.
        if published==False:
            vpsetup = fortpath.replace(fortpath.split('/')[-1],'vp_setup.dat')
            if os.path.exists(vpsetup):
                for line in numpy.loadtxt(vpsetup,dtype=str,delimiter='\n',comments='!'):
                    if 'daoaun'in line:
                        daoaun = float(line.split()[1])
        reg,alpha,error = read_fort26(fortpath.replace('.18','.26'),daoaun)
        fort18 = numpy.loadtxt(fortpath,dtype='str',delimiter='\n')
        for i in range(len(fort18)-1,0,-1):
            if 'statistics for whole fit:' in fort18[i] and flag==1:
                n     = float(fort18[i+2].split()[3])
                df    = float(fort18[i+2].split()[4])
                flag  = 2
            if 'arameter errors:' in fort18[i] and flag==2:
                line  = fort18[i+1+reg]
                s     = 1 if len(line.split()[0])==1 else 0
                if 'q' in str(line.split()[4+s]) and '*' not in str(line.split()[4+s]):
                    error = float(line.split()[4+s].split('q')[0])*daoaun/1e-5
                flag = 3
            if 'chi-squared :' in fort18[i] and flag==3:
                chisq_nu = float(fort18[i].split()[2])
                chisq = float(fort18[i].split()[-3].replace(',',''))
                line  = fort18[i+2+reg]
                s     = 1 if len(line.split()[0])==1 else 0
                if 'q' in str(line.split()[4+s]) and '*' not in str(line.split()[4+s]):
                    alpha = float(line.split()[4+s].split('q')[0])*daoaun/1e-5
                break
    # Create object and include results
    results = type('results', (object,), {})()
    results.chisq_nu=chisq_nu
    results.chisq=chisq
    results.alpha=alpha
    results.error=error
    results.df=df
    results.n=n
    return results

def store_data(fortpath,headpath):
    '''
    Read data from fort.13 and header files and store them in arrays.

    Parameters
    ----------
    fortpath : str
      Path to input fort.13 file
    headpath : str
      Path to input header.dat file
    '''
    read_head = numpy.loadtxt(headpath,dtype='str',delimiter='\n',ndmin=1)
    read_fort = open(fortpath,'r')
    read_fort = [line.strip() for line in read_fort]
    fort_header_old  = numpy.empty((0,8))
    fort_content_old = numpy.empty((0,8))
    i = flag = comp = twoq = 0
    while i < len(read_fort):
        if flag==2 and (read_fort[i]=='' or (read_fort[i].split()[0]=='>>' and read_fort[i].split()[-3]=='1')):
            break
        if read_fort[i]=='*':
            flag = flag+1
            i = offset = i + 1
        if flag==1 and read_fort[i][0]!='!':
            vals = read_fort[i].replace('!',' ').split()
            val0 = 'data/'+vals[0].replace('data/','').replace('../','')
            val1 = int(vals[1])
            val2 = '%.2f'%float(vals[2])
            val3 = '%.2f'%float(vals[3])
            val4 = vals[4].split('=')[0]+'='+str('%5.8f'%float(vals[4].split('=')[1]))
            val5 = read_head[i-offset].split()[0]
            val6 = '' if len(read_head[i-offset].split())==1 else read_head[i-offset].split()[1]
            val7 = '' if len(read_head[i-offset].split())==1 else read_head[i-offset].split()[2]
            fort_header_old = numpy.vstack((fort_header_old,[val0,val1,val2,val3,val4,val5,val6,val7]))
        if flag==2 and read_fort[i][0]!='!':
            vals = read_fort[i].split()
            val0 = vals[0]+' '+vals[1] if len(vals[0])==1 else vals[0]
            k = 1 if len(vals[0])==1 else 0
            val1 = '%.5f'%float(vals[k+1][:-2]+re.compile(r'[^\d.-]+').sub('',vals[k+1][-2:]))
            if '--colfix' in sys.argv:
                val1 = val1+'FF'
            else:
                val1 = val1+" ".join(re.findall("[a-zA-Z]+",vals[k+1][-2:]))
            zabs = float(vals[k+2][:-2]+re.compile(r'[^\d.-]+').sub('',vals[k+2][-2:]))
            val2 = '%.7f'%zabs
            val2 = val2+" ".join(re.findall("[a-zA-Z]+",vals[k+2][-2:]))
            val3 = '%.4f'%float(vals[k+3][:-2]+re.compile(r'[^\d.-]+').sub('',vals[k+3][-2:]))
            val3 = val3+" ".join(re.findall("[a-zA-Z]+",vals[k+3][-2:]))
            div  = 10**(-6) if 'E-0' in vals[k+4] else 1
            val4 = '0.000' if '*' in vals[k+4] else '%.3f'%(float(vals[k+4][:-2]+re.compile(r'[^\d.-]+').sub('',vals[k+4][-2:]))/div)
            val4 = val4+" ".join(re.findall("[a-zA-Z]+",vals[k+4][-2:]))
            twoq = 1 if vals[k+4][-2]=='q' and twoq==0 else twoq
            val5 = '%.2f'%float(vals[k+5][:-2]+re.compile(r'[^\d.-]+').sub('',vals[k+5][-2:]))
            val5 = val5+" ".join(re.findall("[a-zA-Z]+",vals[k+5][-2:]))
            val6 = '%.2E'%float(vals[k+6][:-2]+re.compile(r'[^\d.-]+').sub('',vals[k+6][-2:]))
            val6 = val6+" ".join(re.findall("[a-zA-Z]+",vals[k+6][-2:]))
            val7 = str(int(float(vals[k+7][:-2]+re.compile(r'[^\d.-]+').sub('',vals[k+7][-2:]))))
            val7 = val7+" ".join(re.findall("[a-zA-Z]+",vals[k+7][-2:]))
            fort_content_old = numpy.vstack((fort_content_old,[val0,val1,val2,val3,val4,val5,val6,val7]))
            comp = comp+1 if vals[0] not in ['H','>>','<<','<>','__','??'] else comp
        i = i + 1
    return fort_header_old,fort_content_old,comp,twoq

def fort_update(fort_header_old,fort_content_old,atom,qso,sample,slope2fit,mgisotope=False,
                shape='slope',whitmore=False,arm=None,explist=None,settings=None,stdev=0):
    
    # Initialize parameters
    qvar = numpy.empty((0,2))
    qini,qend,qred,qblue,olap,ovlp = [],[],[],[],[],'-'
    # Initialize arm regions (deprecated, originally used in get_shift module)
    #regtotal = regboth = regblue = regred = 0
    # Store selected fort.13 header, and shifts
    store_shift = []
    fort_header_new = numpy.empty((0,8))
    for i in range (len(fort_header_old)):
        armflag = None
        trans = fort_header_old[i,-3]
        wrest = float(atomic_info(atom,trans)[1])
        wave1 = float(fort_header_old[i,2])
        wave2 = float(fort_header_old[i,3])
        shift = getshift(wave1,wave2,slope2fit,qso,sample,explist,settings,shape,whitmore,stdev)
        if sample.startswith('UV'):
            if arm==None or armflag==arm or armflag in arm:
                fort_header_new = numpy.vstack((fort_header_new,fort_header_old[i]))
                store_shift.append(shift)
                if fort_header_old[i,-2]!='external': qvar=numpy.vstack((qvar,[trans,wrest]))
                if fort_header_old[i,-2]=='overlap': ovlp='o'
            qini.append(wrest)
            if armflag=='blue':
                qblue.append(wrest)
                qend.append(wrest)
            if armflag=='red':
                qred.append(wrest)
                qend.append(wrest)
            if armflag=='overlap':
                olap.append(fort_header_old[i,-3])
        else:
            fort_header_new = numpy.vstack((fort_header_new,fort_header_old[i]))
            store_shift.append(shift)
            if fort_header_old[i,-2]!='external':
                qvar = numpy.vstack((qvar,[fort_header_old[i,-3],wrest]))
            if fort_header_old[i,-2]=='overlap':
                ovlp = 'o'
    if len(qvar)==0:
        qmin = qmax = qvar
    else:
        qlist     = numpy.array(qvar[:,1],dtype='float')
        qlist_min = numpy.where(qlist==min(qlist))[0][0]
        qlist_max = numpy.where(qlist==max(qlist))[0][0]
        qmin = qvar[qlist_min,0].replace('_','')
        qmax = qvar[qlist_max,0].replace('_','')
        qvar = str(int(abs(max(qlist)-min(qlist))))
    # Store selected fort.13 content, and shift components
    fort_content_new = numpy.empty((0,8))
    mgcomps = numpy.empty((0,8))
    for i in range (len(fort_content_old)):
        if mgisotope and 'Mg' in fort_content_old[i,0]:
            mgcomps = numpy.vstack((mgcomps,fort_content_old[i]))
        else:
            fort_content_new = numpy.vstack((fort_content_new,fort_content_old[i]))        
    # Implement Mg and Aw components
    for p in range (len(mgcomps)):
        fort_content_new = numpy.vstack((fort_content_new,mgcomps[p]))        
        fort_content_new[-1,1] = re.compile(r'[^\d.-]+').sub('',fort_content_new[-1,1])+'x'
    for p in range (len(mgcomps)):
        fort_content_new = numpy.vstack((fort_content_new,mgcomps[p]))
        fort_content_new[-1,0] = fort_content_new[-1,0].replace('Mg','Aw')
        if p==0 and len(mgcomps)>1:
            fort_content_new[-1,1] = re.compile(r'[^\d.-]+').sub('',fort_content_new[-1,1])+'%'
        else:
            fort_content_new[-1,1] = re.compile(r'[^\d.-]+').sub('',fort_content_new[-1,1])+'X'
        fort_content_new[-1,2] = fort_content_new[-1,2].upper()
        fort_content_new[-1,3] = fort_content_new[-1,3].upper()
        fort_content_new[-1,4] = fort_content_new[-1,4].upper()
        fort_content_new[-1,5] = fort_content_new[-1,5].upper()
    # Implement fix shift values in fort arrays
    if round(slope2fit,3)!=0:
        for p in range (len(store_shift)):
            shift = ['>>','1.00000FF','0.0000000FF',store_shift[p]+'FF','0.000FF','0.00','0.00E+00',p+1]
            fort_content_new = numpy.vstack((fort_content_new,shift))
    fort_header_new = fort_header_new
    fort_content_new = fort_content_new
    return fort_header_new,fort_content_new,qred,qblue

def get_results(therfort=None,turbfort=None,original=False):
    """
    Extract thermal and turbulent results and calculate
    weighted results using method-of-moments estimator.
    """
    chisq=alpha=error=df='-'
    ther = read_fort18(therfort)
    turb = read_fort18(turbfort)
    if turb.chisq!='-' and ther.chisq!='-':
        k         = ther.n - ther.df
        ther_AICc = ther.chisq + 2*k + 2*k*(k+1)/(ther.n-k-1)
        k         = turb.n - turb.df
        turb_AICc = turb.chisq + 2*k + 2*k*(k+1)/(turb.n-k-1)
        csmin     = min([ther_AICc,turb_AICc])
        k1        = math.exp(-(ther_AICc-csmin)/2)
        k2        = math.exp(-(turb_AICc-csmin)/2)
        k         = k1 + k2
        k1        = k1/k
        k2        = k2/k
        df        = k1 * ther.df    + k2 * turb.df
        chisq     = k1 * ther.chisq + k2 * turb.chisq
        alpha     = k1 * ther.alpha + k2 * turb.alpha
        error     = numpy.sqrt(k1*ther.error**2 + \
                               k2*turb.error**2 + \
                               k1*ther.alpha**2 + \
                               k2*turb.alpha**2 - \
                               alpha**2)
    # Create object and include results
    mom = type('mom', (object,), {})()
    mom.chisq=chisq
    mom.alpha=alpha
    mom.error=error
    mom.df=df
    return ther,turb,mom

def prepare_model(model2fit,header,fort13,simulation):
    if '--finalmodel' in sys.argv:
        fort2fit = model+'_ini.26'
        # Create new fort.13 and header files
        write_header = open('header.dat','w')
        write_fort   = open(fort2fit,'w')
        os.system('rm '+model2fit+'_ini.13')
        for i in range (len(header)):
            datalength = [len(header[k,0]) for k in range (len(header))]
            datalength = "{0:<"+str(max(datalength))+"}"
            write_fort.write('%% ')
            write_fort.write(datalength.format(header[i,0])+' ')
            write_fort.write('{0:>5}'.format(header[i,1])+' ')
            write_fort.write('{0:>10}'.format('%.2f'%float(header[i,2]))+' ')
            write_fort.write('{0:>10}'.format('%.2f'%float(header[i,3]))+' ')
            write_fort.write('{0:<17}'.format(header[i,4])+' ! ')
            write_fort.write('{0:<15}'.format(header[i,5])+' ')
            write_header.write('{0:<15}'.format(header[i,5])+' ')
            if header[i,6]!='':
                write_fort.write('{0:<15}'.format(header[i,6])+' ')
                write_header.write('{0:<15}'.format(header[i,6])+' ')
                write_fort.write('{0:<15}'.format(header[i,7]))
                write_header.write('{0:<15}'.format(header[i,7]))
            write_header.write('\n')
            write_fort.write('\n')
        line26 = numpy.loadtxt(fitdir+system+'/original/'+model2fit+'.26',dtype='str',delimiter='\n')
        for i in range (len(line26)):
            if line26[i][0:2]!='%%':
                write_fort.write(line26[i]+'\n')
        write_header.close()
        write_fort.close()
    else:
        fort2fit = model2fit+'_ini.13'
        # Create new fort.13 and header files
        write_header = open('header.dat','w')
        write_fort   = open(fort2fit,'w')
        write_fort.write('   *\n')
        for i in range (len(header)):
            datalength = [len(header[k,0]) for k in range (len(header))]
            datalength = "{0:<"+str(max(datalength))+"}"
            write_fort.write(datalength.format(header[i,0])+' ')
            write_fort.write('{0:>5}'.format(header[i,1])+' ')
            write_fort.write('{0:>10}'.format('%.2f'%float(header[i,2]))+' ')
            write_fort.write('{0:>10}'.format('%.2f'%float(header[i,3]))+' ')
            write_fort.write('{0:<17}'.format(header[i,4])+' ! ')
            write_fort.write('{0:<15}'.format(header[i,5])+' ')
            write_header.write('{0:<15}'.format(header[i,5])+' ')
            if header[i,6]!='':
                write_fort.write('{0:<15}'.format(header[i,6])+' ')
                write_header.write('{0:<15}'.format(header[i,6])+' ')
                write_fort.write('{0:<15}'.format(header[i,7]))
                write_header.write('{0:<15}'.format(header[i,7]))
            write_header.write('\n')
            write_fort.write('\n')
        write_fort.write('  *\n')
        for i in range (len(fort13)):
            val = fort13[i]
            write_fort.write('   '+'{0:<5}'.format(val[0])+' ')
            write_fort.write('{0:>5}'.format(val[1].split('.')[0])+'.'+'{0:<7}'.format(val[1].split('.')[1])+' ')
            write_fort.write('{0:>3}'.format(val[2].split('.')[0])+'.'+'{0:<9}'.format(val[2].split('.')[1])+' ')
            write_fort.write('{0:>4}'.format(val[3].split('.')[0])+'.'+'{0:<6}'.format(val[3].split('.')[1])+' ')
            if simulation==False and '--noalpha' in sys.argv:
                write_fort.write('    0.000FF  ')
            else:
                write_fort.write('{0:>5}'.format(val[4].split('.')[0])+'.'+'{0:<6}'.format(val[4].split('.')[1])+' ')
            write_fort.write('{0:>5}'.format(val[5].split('.')[0])+'.'+'{0:<2}'.format(val[5].split('.')[1])+' ')
            if model2fit=='turbulent' and val[2][-1].islower():
                write_fort.write('  1.00E+00 ')
            else:
                write_fort.write('  0.00E+00 ')
            write_fort.write('{0:>2}'.format(val[7])+' ! ')
            write_fort.write('{0:>4}'.format(i+1))
            write_fort.write('\n')
        write_header.close()
        write_fort.close()



c = 299792.458
outliers = ['J194454+770552/2.84330/HIRES_sargent',
            'J220852-194400/1.01720/HIRES_sargent',
            'J220852-194400/2.07620/HIRES_sargent',
            'J000448-415728/1.54190/UVES_squader']

#home        = os.getenv('HOME')+'/ASTRO/analysis/alpha'
#home        = os.getenv('HOME')+'/Volumes/ASTRO_HOME/ASTRO/analysis/alpha' if '--server'   in sys.argv else home
#home        = '/media/removable/ASTRO/analysis/alpha'                      if '--external' in sys.argv else home
#home        = '/short/gd5/ASTRO/analysis/alpha'                            if '--raijin'   in sys.argv else home
#alphapath   = '/home/561/vxd561/ASTRO/code/'                               if '--raijin'   in sys.argv else ''
#vpfitpath   = '/home/561/vxd561/ASTRO/code/vpfit10/'                       if '--raijin'   in sys.argv else ''
#fitdir      = home+'/systems/'
#wdrep       = home+'/systems/G191-B2B/'
#Crep        = home+'/dipole/'
#Rrep        = home+'/alpha_analysis/'
#rundir      = home+'/runs/'
#uveshead    = home+'/headers/'
#compdir     = home+'/multiplots/compare/'
#here        = os.getenv('PWD')
#pathdata    = os.path.abspath(__file__).rsplit('/',1)[0] + '/data/'
#pathdata    = os.path.dirname(os.path.realpath(__file__)) + '/data/'
#skipabs     = []
#discard     = []
#keck        = ['HIRES_churchill','HIRES_sargent','HIRES_prochaska','HIRES_outram']
#vlt         = ['UVES_squader']
#murphy      = ['HIRES_murphy_A','HIRES_murphy_B1','HIRES_murphy_B2','HIRES_murphy_C']
#degensys    = ['J111113-080402/3.60770/UVES_squader',
#                 'J034943-381031/3.02470/HIRES_prochaska']
#badwhit     = ['J200324-325145/2.03290/UVES_squader']
#clippeduves = ['J104032-272749/1.38610/UVES_squader',
#                 'J010311+131617/2.30920/UVES_squader',
#                 'J234628+124900/2.17130/UVES_squader',
#                 'J010311+131617/1.79750/UVES_squader',
#                 'J044017-433308/2.04820/UVES_squader',
#                 'J055246-363727/1.74750/UVES_squader',
#                 'J040718-441014/2.59480/UVES_squader']
#done       = False
#alphara    = 17.3
#alphadec   = -61
#uvesblue   = 3900.
#uvesred    = 5800.
#zsample    = 'all'
#zcut       = 1.6
#wd_slope   = numpy.arange(-15,15.1,0.1)
#hires_amp  = numpy.arange(0,400,50)
#args         = numpy.array(sys.argv, dtype='str')
#slope      = 0    if '--slope'    not in sys.argv else float(args[numpy.where(args=='--slope'   )[0][0]+1])
#minslope   = None if '--minslope' not in sys.argv else float(args[numpy.where(args=='--minslope')[0][0]+1])
#maxslope   = None if '--maxslope' not in sys.argv else float(args[numpy.where(args=='--maxslope')[0][0]+1])
#step       = None if '--step'     not in sys.argv else float(args[numpy.where(args=='--step'    )[0][0]+1])
#operation  = None if len(sys.argv)==1 else sys.argv[1]
#clean      = '--clean'     in sys.argv
#compress   = '--compress'  in sys.argv
#constrain  = '--constrain' in sys.argv
#display    = '--display'   in sys.argv
#display2   = '--display2'  in sys.argv
#expind     = '--expind'    in sys.argv  
#fit        = '--fit'       in sys.argv
#fit2       = '--fit2'      in sys.argv
#loop       = '--loop'      in sys.argv
#mgisotope  = '--mgisotope' in sys.argv
#murphy     = '--murphy'    in sys.argv
#newatom    = '--newatom'   in sys.argv
#noalpha    = '--noalpha'   in sys.argv
#original   = '--original'  in sys.argv
#previous   = '--previous'  in sys.argv
#reset      = '--reset'     in sys.argv
#show       = '--show'      in sys.argv
#tbs        = '--tbs'       in sys.argv
#whitmore   = '--whitmore'  in sys.argv
#binning    = 12          if '--bin'        not in sys.argv else   int(args[numpy.where(args=='--bin'       )[0][0]+1])
#order      = 3           if '--order'      not in sys.argv else   int(args[numpy.where(args=='--order'     )[0][0]+1])
#repeat     = 1           if '--repeat'     not in sys.argv else   int(args[numpy.where(args=='--repeat'    )[0][0]+1])
#ntrans     = 5           if '--ntrans'     not in sys.argv else   int(args[numpy.where(args=='--ntrans'    )[0][0]+1])
#stdev      = 0           if '--stdev'      not in sys.argv else float(args[numpy.where(args=='--stdev'     )[0][0]+1])
#chisq      = 1.E-5       if '--chisq'      not in sys.argv else float(args[numpy.where(args=='--chisq'     )[0][0]+1])
#snr        = None        if '--snr'        not in sys.argv else float(args[numpy.where(args=='--snr'       )[0][0]+1])
#threshold  = None        if '--threshold'  not in sys.argv else float(args[numpy.where(args=='--threshold' )[0][0]+1])
#wmin       = None        if '--wmin'       not in sys.argv else float(args[numpy.where(args=='--wmin'      )[0][0]+1])
#wmax       = None        if '--wmax'       not in sys.argv else float(args[numpy.where(args=='--wmax'      )[0][0]+1])
#xmin       = None        if '--xmin'       not in sys.argv else float(args[numpy.where(args=='--xmin'      )[0][0]+1])
#xmax       = None        if '--xmax'       not in sys.argv else float(args[numpy.where(args=='--xmax'      )[0][0]+1])
#ymin       = None        if '--ymin'       not in sys.argv else float(args[numpy.where(args=='--ymin'      )[0][0]+1])
#ymax       = None        if '--ymax'       not in sys.argv else float(args[numpy.where(args=='--ymax'      )[0][0]+1])
#mode       = None        if '--mode'       not in sys.argv else   str(args[numpy.where(args=='--mode'      )[0][0]+1])
#compare    = None        if '--compare'    not in sys.argv else   str(args[numpy.where(args=='--compare'   )[0][0]+1])
#custom     = None        if '--custom'     not in sys.argv else   str(args[numpy.where(args=='--custom'    )[0][0]+1])
#sample     = None        if '--sample'     not in sys.argv else   str(args[numpy.where(args=='--sample'    )[0][0]+1])
#tarfile    = None        if '--include'    not in sys.argv else   str(args[numpy.where(args=='--include'   )[0][0]+1])
#instrument = None        if '--instrument' not in sys.argv else   str(args[numpy.where(args=='--instrument')[0][0]+1])
#simulation = None        if '--simulation' not in sys.argv else   str(args[numpy.where(args=='--simulation')[0][0]+1])
#model      = None        if '--model'      not in sys.argv else   str(args[numpy.where(args=='--model'     )[0][0]+1])
#system     = None        if '--system'     not in sys.argv else   str(args[numpy.where(args=='--system'    )[0][0]+1])
#selection  = None        if '--selection'  not in sys.argv else   str(args[numpy.where(args=='--selection' )[0][0]+1])
#path       = '.'         if '--path'       not in sys.argv else   str(args[numpy.where(args=='--path'      )[0][0]+1])
#plot       = 'all'       if '--plot'       not in sys.argv else   str(args[numpy.where(args=='--plot'      )[0][0]+1])
#vpversion  = '10'        if '--vpfit'      not in sys.argv else   str(args[numpy.where(args=='--vpfit'     )[0][0]+1])
#arm        = None        if '--arm'        not in sys.argv else   str(args[numpy.where(args=='--arm'       )[0][0]+1])
#shape      = 'slope'     if '--shape'      not in sys.argv else   str(args[numpy.where(args=='--shape'     )[0][0]+1])
#vpcommand  = vpfitpath + 'vpfit' + vpversion
#vpfsetup   = 'vp_setup_king.dat' if vpversion=='9.5-king' else 'vp_setup_hires.dat' if instrument=='hires' else 'vp_setup_uves.dat'
#vpfsetup   = numpy.loadtxt(pathdata+vpfsetup,dtype=str,delimiter='\n',comments='!')
#vpfsetup   = numpy.hstack((vpfsetup,['chisqthres %.E 2 %.E'%(chisq,chisq)])) if vpversion!='9.5-king' else vpfsetup
#distortion = '0.000' if round(slope,3)==0 else 'm%.3f'%abs(slope) if slope<0 else 'p%.3f'%abs(slope)
#deviation  = '%.2f'%stdev if expind else '0.00'
##test       = 'v' + vpversion + '_chisq%.E'%chisq
##test       = test+'_'+arm            if instrument=='uves'  and '--simulation' not in sys.argv    else test
##test       = test+'_'+shape          if instrument=='hires' and (slope!=0 or distsep!=0)      else test
##test       = test+'_step%.E'%distsep if distsep!=0                                                else test
##test       = test+'_prev'              if '--previous'   in sys.argv and (slope!=0 or distsep!=0) else test
##test       = test+'_jw'                if '--whitmore'   in sys.argv and (slope!=0 or distsep!=0) else test
##test       = test+'_null'              if '--simulation' not in sys.argv and '--noalpha' in sys.argv  else test
##test       = test+'_new'               if '--newatom'    in sys.argv                                  else test
##test       = test+'_murphy'            if '--murphy'     in sys.argv                                  else test
##test       = test+'_mg'                if '--mgisotope'  in sys.argv                                  else test
##test = test+'_noalpha'         if '--noalpha'     in sys.argv    else test
##test = test+'_colfix'          if '--colfix'      in sys.argv    else test
##test = test+'_fddhires'        if '--fdd'         in sys.argv    else test
##test = test+'_fddhires2'       if '--fdd2'        in sys.argv    else test
##test = test+'_final'           if '--finalmodel'  in sys.argv    else test
#modflag = ''
##modflag = modflag+'_'+str(totrep)+'-0'            if totrep!=0 and sys.argv[1]=='run' else modflag
##modflag = modflag+'_'+str(totrep)+'-'+str(totrep) if totrep!=0 and sys.argv[1]!='run' else modflag
#if '--test' in sys.argv:
#    k = numpy.where(args=='--test')[0][0]
#    test = args[k+1]
#if os.getenv('HOME')=='/home/561/vxd561':
#    curlist = numpy.genfromtxt(pathdata+'allsys.dat',names=True,dtype=object,comments='!')
#    distres = numpy.genfromtxt(pathdata+'sims.dat',names=True,dtype=object,comments='!')
